@@ -70,7 +70,7 @@ def prepare_windows_env(env):
     if env.get('PYTHON'):
         return  # Already manually set by user.
 
-    if (2, 6) <= sys.version_info[:2] <= (2, 7):
+    if sys.version_info[:2] == (2, 7):
         env['PYTHON'] = sys.executable
         return  # The current executable is fine.
 
@@ -84,82 +84,72 @@ def prepare_windows_env(env):
         sub = subprocess.Popen(cmd, stdout=subprocess.PIPE)
         stdout, _ = sub.communicate()
         version = ast.literal_eval(stdout.decode('utf-8').strip())
-        if (2, 6) <= version <= (2, 7):
+        if version == (2, 7):
             return  # Python on PATH is fine
     except OSError:
         pass
 
     # Check default install locations
-    for v in ('26', '27'):
-        path = os.path.join('%SYSTEMDRIVE%', 'Python%s' % v, 'python.exe')
-        path = os.path.expandvars(path)
-        if os.path.isfile(path):
-            log.info('Using "%s" to build libuv...' % path)
-            env['PYTHON'] = path
-            return
-
-    raise DistutilsError('No appropriate Python version found. An '
-                         'installation of 2.6 or 2.7 is required to '
-                         'build libuv. You can set the environment '
-                         'variable "PYTHON" to point to a custom '
-                         'installation location.')
+    path = os.path.join('%SYSTEMDRIVE%', 'Python27', 'python.exe')
+    path = os.path.expandvars(path)
+    if os.path.isfile(path):
+        log.info('Using "%s" to build libuv...' % path)
+        env['PYTHON'] = path
+    else:
+        raise DistutilsError('No appropriate Python version found. An '
+                             'installation of 2.7 is required to '
+                             'build libuv. You can set the environment '
+                             'variable "PYTHON" to point to a custom '
+                             'installation location.')
 
 
 class libuv_build_ext(build_ext):
     libuv_dir      = os.path.join('deps', 'libuv')
     libuv_repo     = 'https://github.com/libuv/libuv.git'
-    libuv_branch   = 'v0.10'
-    libuv_revision = '0bcac64'
+    libuv_branch   = 'v1.x'
+    libuv_revision = '0a8e813'
     libuv_patches  = []
 
     user_options = build_ext.user_options
     user_options.extend([
         ("libuv-clean-compile", None, "Clean libuv tree before compilation"),
-        ("libuv-force-fetch", None, "Remove libuv (if present) and fetch it again")
+        ("libuv-force-fetch", None, "Remove libuv (if present) and fetch it again"),
+        ("libuv-verbose-build", None, "Print output of libuv build process")
     ])
     boolean_options = build_ext.boolean_options
-    boolean_options.extend(["libuv-clean-compile", "libuv-force-fetch"])
+    boolean_options.extend(["libuv-clean-compile", "libuv-force-fetch", "libuv-verbose-build"])
 
     def initialize_options(self):
         build_ext.initialize_options(self)
         self.libuv_clean_compile = 0
         self.libuv_force_fetch = 0
+        self.libuv_verbose_build = 0
 
     def build_extensions(self):
-        if self.compiler.compiler_type == 'mingw32':
-            # Dirty hack to avoid linking with more than one C runtime when using MinGW
-            # Note that this hack forces the compilation to use the old MSVCT
-            self.compiler.dll_libraries = [lib for lib in self.compiler.dll_libraries if not lib.startswith('msvcr')]
         self.force = self.libuv_force_fetch or self.libuv_clean_compile
-        if self.compiler.compiler_type == 'msvc':
+        if sys.platform == 'win32':
             self.libuv_lib = os.path.join(self.libuv_dir, 'Release', 'lib', 'libuv.lib')
         else:
-            self.libuv_lib = os.path.join(self.libuv_dir, 'libuv.a')
+            self.libuv_lib = os.path.join(self.libuv_dir, '.libs', 'libuv.a')
         self.get_libuv()
         # Set compiler options
-        if self.compiler.compiler_type == 'mingw32':
-            self.compiler.add_library_dir(self.libuv_dir)
-            self.compiler.add_library('uv')
-        else:
-            self.extensions[0].extra_objects.extend([self.libuv_lib])
+        self.extensions[0].extra_objects.extend([self.libuv_lib])
         self.compiler.add_include_dir(os.path.join(self.libuv_dir, 'include'))
         if sys.platform.startswith('linux'):
             self.compiler.add_library('rt')
-        elif sys.platform == 'darwin':
-            self.extensions[0].extra_link_args.extend(['-framework', 'CoreServices'])
         elif sys.platform == 'win32':
             self.extensions[0].define_macros.append(('WIN32', 1))
-            if self.compiler.compiler_type == 'msvc':
-                self.extensions[0].extra_link_args.extend(['/NODEFAULTLIB:libcmt', '/LTCG'])
-                self.compiler.add_library('advapi32')
+            self.extensions[0].extra_link_args.extend(['/NODEFAULTLIB:libcmt', '/LTCG'])
+            self.compiler.add_library('advapi32')
             self.compiler.add_library('iphlpapi')
             self.compiler.add_library('psapi')
             self.compiler.add_library('ws2_32')
+        elif sys.platform.startswith('freebsd'):
+            self.compiler.add_library('kvm')
         build_ext.build_extensions(self)
 
     def get_libuv(self):
         #self.debug_mode =  bool(self.debug) or hasattr(sys, 'gettotalrefcount')
-        win32_msvc = self.compiler.compiler_type=='msvc'
         def download_libuv():
             log.info('Downloading libuv...')
             makedirs(self.libuv_dir)
@@ -175,12 +165,14 @@ class libuv_build_ext(build_ext):
             env = os.environ.copy()
             env['CFLAGS'] = ' '.join(x for x in (cflags, env.get('CFLAGS', None), env.get('ARCHFLAGS', None)) if x)
             log.info('Building libuv...')
-            if win32_msvc:
+            if sys.platform == 'win32':
                 prepare_windows_env(env)
                 libuv_arch = {'32bit': 'x86', '64bit': 'x64'}[platform.architecture()[0]]
-                exec_process(['cmd.exe', '/C', 'vcbuild.bat', libuv_arch, 'release'], cwd=self.libuv_dir, env=env, shell=True)
+                exec_process(['cmd.exe', '/C', 'vcbuild.bat', libuv_arch, 'release'], cwd=self.libuv_dir, env=env, shell=True, silent=not self.libuv_verbose_build)
             else:
-                exec_process(['make', 'libuv.a'], cwd=self.libuv_dir, env=env)
+                exec_process(['sh', 'autogen.sh'], cwd=self.libuv_dir, env=env, silent=not self.libuv_verbose_build)
+                exec_process(['./configure'], cwd=self.libuv_dir, env=env, silent=not self.libuv_verbose_build)
+                exec_process(['make'], cwd=self.libuv_dir, env=env, silent=not self.libuv_verbose_build)
         if self.libuv_force_fetch:
             rmtree('deps')
         if not os.path.exists(self.libuv_dir):
@@ -193,13 +185,14 @@ class libuv_build_ext(build_ext):
             build_libuv()
         else:
             if self.libuv_clean_compile:
-                if win32_msvc:
+                if sys.platform == 'win32':
                     env = os.environ.copy()
                     prepare_windows_env(env)
                     exec_process(['cmd.exe', '/C', 'vcbuild.bat', 'clean'], cwd=self.libuv_dir, env=env, shell=True)
                     rmtree(os.path.join(self.libuv_dir, 'Release'))
                 else:
                     exec_process(['make', 'clean'], cwd=self.libuv_dir)
+                    exec_process(['make', 'distclean'], cwd=self.libuv_dir)
             if not os.path.exists(self.libuv_lib):
                 log.info('libuv needs to be compiled.')
                 build_libuv()
@@ -232,4 +225,6 @@ class libuv_sdist(sdist):
 
         log.info('Downloading gyp...')
         exec_process(['git', 'clone', self.gyp_repo, self.gyp_dir])
+        rmtree(os.path.join(self.gyp_dir, 'test'))
         rmtree(os.path.join(self.gyp_dir, '.git'))
+
