@@ -1,14 +1,12 @@
 
 static void
-on_check_callback(uv_check_t *handle, int status)
+pyuv__check_cb(uv_check_t *handle)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
     Check *self;
     PyObject *result;
 
     ASSERT(handle);
-    ASSERT(status == 0);
-
     self = PYUV_CONTAINER_OF(handle, Check, check_h);
 
     /* Object could go out of scope in the callback, increase refcount to avoid it */
@@ -28,7 +26,7 @@ on_check_callback(uv_check_t *handle, int status)
 static PyObject *
 Check_func_start(Check *self, PyObject *args)
 {
-    int r;
+    int err;
     PyObject *tmp, *callback;
 
     tmp = NULL;
@@ -45,9 +43,9 @@ Check_func_start(Check *self, PyObject *args)
         return NULL;
     }
 
-    r = uv_check_start(&self->check_h, on_check_callback);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_CheckError);
+    err = uv_check_start(&self->check_h, pyuv__check_cb);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_CheckError);
         return NULL;
     }
 
@@ -56,6 +54,8 @@ Check_func_start(Check *self, PyObject *args)
     self->callback = callback;
     Py_XDECREF(tmp);
 
+    PYUV_HANDLE_INCREF(self);
+
     Py_RETURN_NONE;
 }
 
@@ -63,20 +63,22 @@ Check_func_start(Check *self, PyObject *args)
 static PyObject *
 Check_func_stop(Check *self)
 {
-    int r;
+    int err;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    r = uv_check_stop(&self->check_h);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_CheckError);
+    err = uv_check_stop(&self->check_h);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_CheckError);
         return NULL;
     }
 
     Py_XDECREF(self->callback);
     self->callback = NULL;
 
+    PYUV_HANDLE_DECREF(self);
+
     Py_RETURN_NONE;
 }
 
@@ -84,7 +86,7 @@ Check_func_stop(Check *self)
 static int
 Check_tp_init(Check *self, PyObject *args, PyObject *kwargs)
 {
-    int r;
+    int err;
     Loop *loop;
 
     UNUSED_ARG(kwargs);
@@ -95,9 +97,9 @@ Check_tp_init(Check *self, PyObject *args, PyObject *kwargs)
         return -1;
     }
 
-    r = uv_check_init(loop->uv_loop, &self->check_h);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(loop->uv_loop, PyExc_CheckError);
+    err = uv_check_init(loop->uv_loop, &self->check_h);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_CheckError);
         return -1;
     }
 
