@@ -1,14 +1,12 @@
 
 static void
-on_idle_callback(uv_idle_t *handle, int status)
+pyuv__idle_cb(uv_idle_t *handle)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
     Idle *self;
     PyObject *result;
 
     ASSERT(handle);
-    ASSERT(status == 0);
-
     self = PYUV_CONTAINER_OF(handle, Idle, idle_h);
 
     /* Object could go out of scope in the callback, increase refcount to avoid it */
@@ -28,7 +26,7 @@ on_idle_callback(uv_idle_t *handle, int status)
 static PyObject *
 Idle_func_start(Idle *self, PyObject *args)
 {
-    int r;
+    int err;
     PyObject *tmp, *callback;
 
     tmp = NULL;
@@ -45,9 +43,9 @@ Idle_func_start(Idle *self, PyObject *args)
         return NULL;
     }
 
-    r = uv_idle_start(&self->idle_h, on_idle_callback);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_IdleError);
+    err = uv_idle_start(&self->idle_h, pyuv__idle_cb);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_IdleError);
         return NULL;
     }
 
@@ -56,6 +54,8 @@ Idle_func_start(Idle *self, PyObject *args)
     self->callback = callback;
     Py_XDECREF(tmp);
 
+    PYUV_HANDLE_INCREF(self);
+
     Py_RETURN_NONE;
 }
 
@@ -63,20 +63,22 @@ Idle_func_start(Idle *self, PyObject *args)
 static PyObject *
 Idle_func_stop(Idle *self)
 {
-    int r;
+    int err;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    r = uv_idle_stop(&self->idle_h);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_IdleError);
+    err = uv_idle_stop(&self->idle_h);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_IdleError);
         return NULL;
     }
 
     Py_XDECREF(self->callback);
     self->callback = NULL;
 
+    PYUV_HANDLE_DECREF(self);
+
     Py_RETURN_NONE;
 }
 
@@ -84,7 +86,7 @@ Idle_func_stop(Idle *self)
 static int
 Idle_tp_init(Idle *self, PyObject *args, PyObject *kwargs)
 {
-    int r;
+    int err;
     Loop *loop;
 
     UNUSED_ARG(kwargs);
@@ -95,9 +97,9 @@ Idle_tp_init(Idle *self, PyObject *args, PyObject *kwargs)
         return -1;
     }
 
-    r = uv_idle_init(loop->uv_loop, &self->idle_h);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(loop->uv_loop, PyExc_IdleError);
+    err = uv_idle_init(loop->uv_loop, &self->idle_h);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_IdleError);
         return -1;
     }
 
