@@ -1,81 +1,51 @@
 
-static Loop *default_loop = NULL;
-
-
-static void
-_loop_cleanup(void)
-{
-    Py_XDECREF(default_loop);
-}
-
-
-static int
-init_loop(Loop *loop, int is_default)
-{
-    uv_loop_t *uv_loop;
-
-    uv_loop = NULL;
-
-    if (is_default) {
-        uv_loop = uv_default_loop();
-    } else {
-        uv_loop = uv_loop_new();
-    }
-
-    if (!uv_loop) {
-        PyErr_NoMemory();
-        return -1;
-    }
-    uv_loop->data = loop;
-
-    loop->uv_loop = uv_loop;
-    loop->is_default = is_default;
-    loop->weakreflist = NULL;
-    loop->excepthook_cb = NULL;
-
-    return 0;
-}
+static PyObject *default_loop = NULL;
 
 
 static PyObject *
 new_loop(PyTypeObject *type, PyObject *args, PyObject *kwargs, int is_default)
 {
+    PyObject *obj;
+    Loop *loop;
+    uv_loop_t *uv_loop;
+
     if ((args && PyTuple_GET_SIZE(args)) || (kwargs && PyDict_Check(kwargs) && PyDict_Size(kwargs))) {
         PyErr_SetString(PyExc_TypeError, "Loop initialization takes no parameters");
         return NULL;
     }
 
-    if (is_default) {
-        if (!default_loop) {
-            default_loop = (Loop *)PyType_GenericNew(type, args, kwargs);
-            if (!default_loop) {
-                return NULL;
-            }
-            if (init_loop(default_loop, True) != 0) {
-                return NULL;
-            }
-            Py_AtExit(_loop_cleanup);
-        }
-        Py_INCREF(default_loop);
-        return (PyObject *)default_loop;
-    } else {
-        Loop *self = (Loop *)PyType_GenericNew(type, args, kwargs);
-        if (!self) {
-            return NULL;
-        }
-        if (init_loop(self, False) != 0) {
-            return NULL;
-        }
-        return (PyObject *)self;
+    obj = PyType_GenericNew(type, args, kwargs);
+    if (!obj) {
+        return NULL;
     }
+
+    loop = (Loop *) obj;
+    if (is_default) {
+        uv_loop = uv_default_loop();
+    } else {
+        uv_loop = &loop->loop_struct;
+    }
+
+    if (uv_loop_init(uv_loop) < 0) {
+        PyErr_SetString(PyExc_RuntimeError, "Error initializing loop");
+        Py_DECREF(obj);
+        return NULL;
+    }
+
+    uv_loop->data = loop;
+    loop->uv_loop = uv_loop;
+    loop->is_default = is_default;
+    loop->weakreflist = NULL;
+    loop->buffer.in_use = False;
+
+    return obj;
 }
 
 
 static PyObject *
 Loop_func_run(Loop *self, PyObject *args)
 {
-    int r;
-    int mode;
+    int mode, r;
 
     mode = UV_RUN_DEFAULT;
 
@@ -92,10 +62,6 @@ Loop_func_run(Loop *self, PyObject *args)
     r = uv_run(self->uv_loop, mode);
     Py_END_ALLOW_THREADS
 
-    if (PyErr_Occurred()) {
-        handle_uncaught_exception(self);
-    }
-
     return PyBool_FromLong((long)r);
 }
 
@@ -123,46 +89,6 @@ Loop_func_update_time(Loop *self)
 }
 
 
-static void
-walk_cb(uv_handle_t* handle, void* arg)
-{
-    PyObject *result;
-    PyObject *callback = (PyObject *)arg;
-    PyObject *obj = (PyObject *)handle->data;
-
-    ASSERT(obj);
-
-    Py_INCREF(obj);
-    result = PyObject_CallFunctionObjArgs(callback, obj, NULL);
-    if (result == NULL) {
-        handle_uncaught_exception(((Handle *)obj)->loop);
-    }
-    Py_DECREF(obj);
-    Py_XDECREF(result);
-}
-
-static PyObject *
-Loop_func_walk(Loop *self, PyObject *args)
-{
-    PyObject *callback;
-
-    if (!PyArg_ParseTuple(args, "O:walk", &callback)) {
-        return NULL;
-    }
-
-    if (!PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable is required");
-        return NULL;
-    }
-
-    Py_INCREF(callback);
-    uv_walk(self->uv_loop, (uv_walk_cb)walk_cb, callback);
-    Py_DECREF(callback);
-
-    Py_RETURN_NONE;
-}
-
-
 static PyObject *
 Loop_func_fileno(Loop *self)
 {
@@ -178,7 +104,7 @@ Loop_func_get_timeout(Loop *self)
 
 
 static void
-threadpool_work_cb(uv_work_t *req)
+pyuv__tp_work_cb(uv_work_t *req)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
     WorkRequest *work_req;
@@ -198,7 +124,7 @@ threadpool_work_cb(uv_work_t *req)
 }
 
 static void
-threadpool_done_cb(uv_work_t *req, int status)
+pyuv__tp_done_cb(uv_work_t *req, int status)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
     WorkRequest *work_req;
@@ -212,8 +138,7 @@ threadpool_done_cb(uv_work_t *req, int status)
 
     if (work_req->done_cb != Py_None) {
         if (status < 0) {
-            uv_err_t err = uv_last_error(req->loop);
-            errorno = PyInt_FromLong((long)err.code);
+            errorno = PyInt_FromLong((long)status);
         } else {
             errorno = Py_None;
             Py_INCREF(Py_None);
@@ -236,7 +161,7 @@ threadpool_done_cb(uv_work_t *req, int status)
 static PyObject *
 Loop_func_queue_work(Loop *self, PyObject *args)
 {
-    int r;
+    int err;
     WorkRequest *work_req;
     PyObject *work_cb, *done_cb;
 
@@ -262,9 +187,9 @@ Loop_func_queue_work(Loop *self, PyObject *args)
         return NULL;
     }
 
-    r = uv_queue_work(self->uv_loop, &work_req->req, threadpool_work_cb, threadpool_done_cb);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(self->uv_loop, PyExc_Exception);
+    err = uv_queue_work(self->uv_loop, &work_req->req, pyuv__tp_work_cb, pyuv__tp_done_cb);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_Exception);
         goto error;
     }
 
@@ -277,11 +202,44 @@ error:
 }
 
 
+static PyObject *
+Loop_func_excepthook(Loop *self, PyObject *args)
+{
+    PyObject *exc, *value, *tb;
+
+    if (!PyArg_ParseTuple(args, "OOO:excepthook", &exc, &value, &tb)) {
+        return NULL;
+    }
+    Py_INCREF(exc);
+    Py_INCREF(value);
+    Py_INCREF(tb);
+    PyErr_Restore(exc, value, tb);
+    PySys_WriteStderr("Unhandled exception in callback\n");
+    PyErr_PrintEx(0);
+    PyErr_Clear();
+    Py_RETURN_NONE;
+}
+
+
 static PyObject *
 Loop_func_default_loop(PyObject *cls)
 {
-    UNUSED_ARG(cls);
-    return new_loop(&LoopType, NULL, NULL, 1);
+    PyTypeObject *type = (PyTypeObject *) cls;
+    if (!default_loop) {
+        default_loop = new_loop((PyTypeObject *)cls, NULL, NULL, True);
+        if (!default_loop) {
+            return NULL;
+        }
+        if (type->tp_init != PyBaseObject_Type.tp_init) {
+            if (type->tp_init(default_loop, PyTuple_New(0), NULL) < 0) {
+                Py_XDECREF(default_loop);
+                default_loop = NULL;
+                return NULL;
+            }
+        }
+    }
+    Py_INCREF(default_loop);
+    return default_loop;
 }
 
 
@@ -297,10 +255,45 @@ Loop_default_get(Loop *self, void *closure)
 }
 
 
+static void
+handles_walk_cb(uv_handle_t* handle, void* arg)
+{
+    PyObject *handles, *obj;
+
+    handles = arg;
+    obj = handle->data;
+
+    ASSERT(obj);
+
+    PyList_Append(handles, obj);
+}
+
+static PyObject *
+Loop_handles_get(Loop *self, void *closure)
+{
+    PyObject *handles;
+    UNUSED_ARG(closure);
+
+    handles = PyList_New(0);
+
+    uv_walk(self->uv_loop, (uv_walk_cb)handles_walk_cb, handles);
+
+    return handles;
+}
+
+
+static PyObject *
+Loop_alive_get(Loop *self, void *closure)
+{
+    UNUSED_ARG(closure);
+    return PyBool_FromLong((long)uv_loop_alive(self->uv_loop));
+}
+
+
 static PyObject *
 Loop_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 {
-    return new_loop(type, args, kwargs, 0);
+    return new_loop(type, args, kwargs, False);
 }
 
 
@@ -325,12 +318,12 @@ Loop_tp_dealloc(Loop *self)
 {
     if (self->uv_loop) {
         self->uv_loop->data = NULL;
-        uv_loop_delete(self->uv_loop);
+        uv_loop_close(self->uv_loop);
     }
     if (self->weakreflist != NULL) {
         PyObject_ClearWeakRefs((PyObject *)self);
     }
-    Loop_tp_clear(self);
+    Py_TYPE(self)->tp_clear((PyObject *)self);
     Py_TYPE(self)->tp_free((PyObject *)self);
 }
 
@@ -374,56 +367,26 @@ Loop_dict_set(Loop *self, PyObject* val, void* c)
 }
 
 
-static PyObject*
-Loop_excepthook_get(Loop *self, void* c)
-{
-    UNUSED_ARG(c);
-    return self->excepthook_cb;
-}
-
-
-static int
-Loop_excepthook_set(Loop *self, PyObject* val, void* c)
-{
-    PyObject* tmp;
-
-    UNUSED_ARG(c);
-
-    if (val == NULL) {
-        PyErr_SetString(PyExc_TypeError, "excepthook may not be deleted");
-        return -1;
-    }
-    if (val != Py_None && !PyCallable_Check(val)) {
-        PyErr_SetString(PyExc_TypeError, "a callable or None is required");
-        return -1;
-    }
-    tmp = self->excepthook_cb;
-    Py_INCREF(val);
-    self->excepthook_cb = val;
-    Py_XDECREF(tmp);
-    return 0;
-}
-
-
 static PyMethodDef
 Loop_tp_methods[] = {
     { "run", (PyCFunction)Loop_func_run, METH_VARARGS, "Run the event loop." },
     { "stop", (PyCFunction)Loop_func_stop, METH_NOARGS, "Stop running the event loop." },
     { "now", (PyCFunction)Loop_func_now, METH_NOARGS, "Return event loop time, expressed in nanoseconds." },
     { "update_time", (PyCFunction)Loop_func_update_time, METH_NOARGS, "Update event loop's notion of time by querying the kernel." },
-    { "walk", (PyCFunction)Loop_func_walk, METH_VARARGS, "Walk all handles in the loop." },
     { "fileno", (PyCFunction)Loop_func_fileno, METH_NOARGS, "Get the loop backend file descriptor." },
     { "get_timeout", (PyCFunction)Loop_func_get_timeout, METH_NOARGS, "Get the poll timeout, or -1 for no timeout." },
     { "default_loop", (PyCFunction)Loop_func_default_loop, METH_CLASS|METH_NOARGS, "Instantiate the default loop." },
     { "queue_work", (PyCFunction)Loop_func_queue_work, METH_VARARGS, "Queue the given function to be run in the thread pool." },
+    { "excepthook", (PyCFunction)Loop_func_excepthook, METH_VARARGS, "Loop uncaught exception handler" },
     { NULL }
 };
 
 
 static PyGetSetDef Loop_tp_getsets[] = {
     {"__dict__", (getter)Loop_dict_get, (setter)Loop_dict_set, NULL},
+    {"alive", (getter)Loop_alive_get, NULL, "Indicates if the loop is still running / alive", NULL},
     {"default", (getter)Loop_default_get, NULL, "Is this the default loop?", NULL},
-    {"excepthook", (getter)Loop_excepthook_get, (setter)Loop_excepthook_set, "Loop uncaught exception handler", NULL},
+    {"handles", (getter)Loop_handles_get, NULL, "Returns a list with all handles in the Loop", NULL},
     {NULL}
 };
 
@@ -469,4 +432,3 @@ static PyTypeObject LoopType = {
     Loop_tp_new,                                                    /*tp_new*/
 };
 
-
