@@ -1,6 +1,7 @@
 
 #include "pyuv.h"
 
+#include "common.c"
 #include "errno.c"
 #include "error.c"
 #include "loop.c"
@@ -20,10 +21,10 @@
 #include "poll.c"
 #include "fs.c"
 #include "process.c"
+#include "dns.c"
 #include "util.c"
 #include "thread.c"
 
-#define LIBUV_VERSION UV_VERSION_MAJOR.UV_VERSION_MINOR-LIBUV_REVISION
 
 #ifdef PYUV_PYTHON3
 static PyModuleDef pyuv_module = {
@@ -38,7 +39,7 @@ static PyModuleDef pyuv_module = {
 /* borrowed from pyev */
 #ifdef PYUV_WINDOWS
 static int
-pyuv_setmaxstdio(void)
+pyuv__setmaxstdio(void)
 {
     if (_setmaxstdio(PYUV_MAXSTDIO) != PYUV_MAXSTDIO) {
         if (errno) {
@@ -63,6 +64,7 @@ init_pyuv(void)
     PyObject *errno_module;
     PyObject *error_module;
     PyObject *fs_module;
+    PyObject *dns_module;
     PyObject *util_module;
     PyObject *thread_module;
 
@@ -70,7 +72,7 @@ init_pyuv(void)
     PyEval_InitThreads();
 
 #ifdef PYUV_WINDOWS
-    if (pyuv_setmaxstdio()) {
+    if (pyuv__setmaxstdio()) {
         return NULL;
     }
 #endif
@@ -115,6 +117,17 @@ init_pyuv(void)
     Py_DECREF(fs_module);
 #endif
 
+    /* DNS module */
+    dns_module = init_dns();
+    if (dns_module == NULL) {
+        goto fail;
+    }
+    PyUVModule_AddObject(pyuv, "dns", dns_module);
+#ifdef PYUV_PYTHON3
+    PyDict_SetItemString(PyImport_GetModuleDict(), pyuv_dns_module.m_name, dns_module);
+    Py_DECREF(dns_module);
+#endif
+
     /* Util module */
     util_module = init_util();
     if (util_module == NULL) {
@@ -157,6 +170,10 @@ init_pyuv(void)
     if (PyType_Ready(&GAIRequestType) < 0) {
         return NULL;
     }
+    GNIRequestType.tp_base = &RequestType;
+    if (PyType_Ready(&GNIRequestType) < 0) {
+        return NULL;
+    }
     WorkRequestType.tp_base = &RequestType;
     if (PyType_Ready(&WorkRequestType) < 0) {
         return NULL;
@@ -181,6 +198,10 @@ init_pyuv(void)
     PyUVModule_AddType(pyuv, "StdIO", &StdIOType);
     PyUVModule_AddType(pyuv, "Process", &ProcessType);
 
+    /* Handle and Stream base classes */
+    PyUVModule_AddType(pyuv, "Handle", &HandleType);
+    PyUVModule_AddType(pyuv, "Stream", &StreamType);
+
     /* Loop.run modes */
     PyModule_AddIntMacro(pyuv, UV_RUN_DEFAULT);
     PyModule_AddIntMacro(pyuv, UV_RUN_ONCE);
@@ -191,6 +212,10 @@ init_pyuv(void)
     PyModule_AddIntMacro(pyuv, UV_LEAVE_GROUP);
     PyModule_AddIntMacro(pyuv, UV_UDP_PARTIAL);
     PyModule_AddIntMacro(pyuv, UV_UDP_IPV6ONLY);
+    PyModule_AddIntMacro(pyuv, UV_UDP_REUSEADDR);
+
+    /* TCP constants */
+    PyModule_AddIntMacro(pyuv, UV_TCP_IPV6ONLY);
 
     /* Process constants */
     PyModule_AddIntMacro(pyuv, UV_PROCESS_SETUID);
