@@ -14,6 +14,8 @@
     #define PYUV_PYTHON3
     #define PyInt_FromSsize_t PyLong_FromSsize_t
     #define PyInt_FromLong PyLong_FromLong
+    #define PyInt_Check PyLong_Check
+    #define PyInt_AsLong PyLong_AsLong
     #define PYUV_BYTES "y"
 #else
     #define PYUV_BYTES "s"
@@ -49,6 +51,15 @@ typedef int Bool;
     #define INLINE inline
 #endif
 
+#if defined(_MSC_VER)
+# include <malloc.h>
+# define STACK_ARRAY(type, name, size) type* name = (type*)_alloca((size) * sizeof(type))
+#else
+# define STACK_ARRAY(type, name, size) type name[size]
+#endif
+
+#define ARRAY_SIZE(a) (sizeof(a) / sizeof((a)[0]))
+
 #define ASSERT(x)                                                           \
     do {                                                                    \
         if (!(x)) {                                                         \
@@ -98,17 +109,16 @@ typedef int Bool;
         }                                                                   \
     } while(0)                                                              \
 
-#define RAISE_UV_EXCEPTION(loop, exc_type)                                          \
+#define RAISE_UV_EXCEPTION(err, exc_type)                                           \
     do {                                                                            \
-        uv_err_t err = uv_last_error(loop);                                         \
-        PyObject *exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));     \
+        PyObject *exc_data = Py_BuildValue("(is)", err, uv_strerror(err));          \
         if (exc_data != NULL) {                                                     \
             PyErr_SetObject(exc_type, exc_data);                                    \
             Py_DECREF(exc_data);                                                    \
         }                                                                           \
     } while(0)                                                                      \
 
-#define RAISE_STREAM_EXCEPTION(handle)                                              \
+#define RAISE_STREAM_EXCEPTION(err, handle)                                         \
     do {                                                                            \
         PyObject *exc_type;                                                         \
         switch ((handle)->type) {                                                   \
@@ -124,22 +134,52 @@ typedef int Bool;
             default:                                                                \
                 ASSERT(0 && "invalid stream handle type");                          \
         }                                                                           \
-        RAISE_UV_EXCEPTION((handle)->loop, exc_type);                               \
+        RAISE_UV_EXCEPTION(err, exc_type);                                          \
     } while(0)                                                                      \
 
+#define PYUV_SET_NONE(x)     \
+    do {                     \
+        Py_INCREF(Py_None);  \
+        (x) = Py_None;       \
+    } while(0)               \
+
 #define PYUV_SLAB_SIZE 65536
 
 
+/* Custom pyuv handle flags */
+#define PYUV__PYREF    (1 << 1)
+
+#define PYUV_HANDLE_INCREF(obj)                        \
+    do {                                               \
+        if (!(HANDLE(obj)->flags & PYUV__PYREF)) {     \
+            HANDLE(obj)->flags |= PYUV__PYREF;         \
+            Py_INCREF(obj);                            \
+        }                                              \
+    } while(0)                                         \
+
+#define PYUV_HANDLE_DECREF(obj)                        \
+    do {                                               \
+        if (HANDLE(obj)->flags & PYUV__PYREF) {        \
+            HANDLE(obj)->flags &= ~PYUV__PYREF;        \
+            Py_DECREF(obj);                            \
+        }                                              \
+    } while(0)                                         \
+
+
 /* Python types definitions */
 
 /* Loop */
 typedef struct {
     PyObject_HEAD
-    PyObject *excepthook_cb;
     PyObject *weakreflist;
     PyObject *dict;
+    uv_loop_t loop_struct;
     uv_loop_t *uv_loop;
     int is_default;
+    struct {
+        char slab[PYUV_SLAB_SIZE];
+        Bool in_use;
+    } buffer;
 } Loop;
 
 static PyTypeObject LoopType;
@@ -148,6 +188,7 @@ static PyTypeObject LoopType;
 typedef struct {
     PyObject_HEAD
     uv_handle_t *uv_handle;
+    int flags;
     Bool initialized;
     PyObject *weakreflist;
     PyObject *dict;
@@ -268,7 +309,6 @@ typedef struct {
     Handle handle;
     uv_poll_t poll_h;
     PyObject *callback;
-    long fd;
 } Poll;
 
 static PyTypeObject PollType;
@@ -362,6 +402,7 @@ typedef struct {
     Bool initialized;
     uv_req_t *req_ptr;
     Loop *loop;
+    PyObject *dict;
 } Request;
 
 static PyTypeObject RequestType;
@@ -375,6 +416,15 @@ typedef struct {
 
 static PyTypeObject GAIRequestType;
 
+/* GNIRequest */
+typedef struct {
+    Request request;
+    uv_getnameinfo_t req;
+    PyObject *callback;
+} GNIRequest;
+
+static PyTypeObject GNIRequestType;
+
 /* WorkRequest */
 typedef struct {
     Request request;
@@ -390,6 +440,13 @@ typedef struct {
     Request request;
     uv_fs_t req;
     PyObject *callback;
+    PyObject *path;
+    PyObject *result;
+    PyObject *error;
+    /* for write requests */
+    Py_buffer view;
+    /* for read requests */
+    uv_buf_t buf;
 } FSRequest;
 
 static PyTypeObject FSRequestType;
@@ -478,6 +535,8 @@ static PyStructSequence_Field interface_addresses_result_fields[] = {
     {"name", ""},
     {"is_internal", ""},
     {"address", ""},
+    {"netmask", ""},
+    {"mac", ""},
     {NULL}
 };
 
@@ -485,7 +544,7 @@ static PyStructSequence_Desc interface_addresses_result_desc = {
     "interface_addresses_result",
     NULL,
     interface_addresses_result_fields,
-    3
+    5
 };
 
 
@@ -527,232 +586,36 @@ static PyStructSequence_Desc cpu_info_times_result_desc = {
 };
 
 
-/* Some helper stuff */
+/* used by getrusage */
+static PyTypeObject RusageResultType;
 
+static PyStructSequence_Field rusage_result_fields[] = {
+    {"ru_utime",        "user time used"},
+    {"ru_stime",        "system time used"},
+    {"ru_maxrss",       "max. resident set size"},
+    {"ru_ixrss",        "shared memory size"},
+    {"ru_idrss",        "unshared data size"},
+    {"ru_isrss",        "unshared stack size"},
+    {"ru_minflt",       "page faults not requiring I/O"},
+    {"ru_majflt",       "page faults requiring I/O"},
+    {"ru_nswap",        "number of swap outs"},
+    {"ru_inblock",      "block input operations"},
+    {"ru_oublock",      "block output operations"},
+    {"ru_msgsnd",       "IPC messages sent"},
+    {"ru_msgrcv",       "IPC messages received"},
+    {"ru_nsignals",     "signals received"},
+    {"ru_nvcsw",        "voluntary context switches"},
+    {"ru_nivcsw",       "involuntary context switches"},
+    {NULL}
+};
 
-/* Add a type to a module */
-static int
-PyUVModule_AddType(PyObject *module, const char *name, PyTypeObject *type)
-{
-    if (PyType_Ready(type)) {
-        return -1;
-    }
-    Py_INCREF(type);
-    if (PyModule_AddObject(module, name, (PyObject *)type)) {
-        Py_DECREF(type);
-        return -1;
-    }
-    return 0;
-}
-
-
-/* Add a type to a module */
-static int
-PyUVModule_AddObject(PyObject *module, const char *name, PyObject *value)
-{
-    Py_INCREF(value);
-    if (PyModule_AddObject(module, name, value)) {
-        Py_DECREF(value);
-        return -1;
-    }
-    return 0;
-}
-
-
-static INLINE int
-pyseq2uvbuf(PyObject *seq, Py_buffer **rviews, uv_buf_t **rbufs, int *rbuf_count)
-{
-    int i, j, buf_count;
-    uv_buf_t *uv_bufs = NULL;
-    Py_buffer *views = NULL;
-    PyObject *data_fast = NULL;
-
-    i = 0;
-    *rviews = NULL;
-    *rbufs = NULL;
-    *rbuf_count = 0;
-
-    if ((data_fast = PySequence_Fast(seq, "argument 1 must be an iterable")) == NULL) {
-        goto error;
-    }
-
-    buf_count = PySequence_Fast_GET_SIZE(data_fast);
-    if (buf_count > INT_MAX) {
-        PyErr_SetString(PyExc_ValueError, "argument 1 is too long");
-        goto error;
-    }
-
-    if (buf_count == 0) {
-        PyErr_SetString(PyExc_ValueError, "argument 1 is empty");
-        goto error;
-    }
-
-    uv_bufs = PyMem_Malloc(sizeof *uv_bufs * buf_count);
-    views = PyMem_Malloc(sizeof *views * buf_count);
-    if (!uv_bufs || !views) {
-        PyErr_NoMemory();
-        goto error;
-    }
-
-    for (i = 0; i < buf_count; i++) {
-        if (!PyArg_Parse(PySequence_Fast_GET_ITEM(data_fast, i), PYUV_BYTES"*;argument 1 must be an iterable of buffer-compatible objects", &views[i])) {
-            goto error;
-        }
-        uv_bufs[i].base = views[i].buf;
-        uv_bufs[i].len = views[i].len;
-    }
-
-    *rviews = views;
-    *rbufs = uv_bufs;
-    *rbuf_count = buf_count;
-    Py_XDECREF(data_fast);
-    return 0;
-
-error:
-    for (j = 0; j < i; j++) {
-        PyBuffer_Release(&views[j]);
-    }
-    PyMem_Free(views);
-    PyMem_Free(uv_bufs);
-    Py_XDECREF(data_fast);
-    return -1;
-}
-
-
-/* parse a Python tuple containing host, port, flowinfo, scope_id into a sockaddr struct */
-static int
-pyuv_parse_addr_tuple(PyObject *addr, struct sockaddr_storage *ss)
-{
-    char *host;
-    int port;
-    unsigned int scope_id, flowinfo;
-    struct in_addr addr4;
-    struct in6_addr addr6;
-    struct sockaddr_in *sa4;
-    struct sockaddr_in6 *sa6;
-
-    flowinfo = scope_id = 0;
-
-    if (!PyTuple_Check(addr)) {
-        PyErr_Format(PyExc_TypeError, "address must be tuple, not %.500s", Py_TYPE(addr)->tp_name);
-        return -1;
-    }
-
-    if (!PyArg_ParseTuple(addr, "si|II", &host, &port, &flowinfo, &scope_id)) {
-        return -1;
-    }
-
-    if (port < 0 || port > 0xffff) {
-        PyErr_SetString(PyExc_OverflowError, "port must be 0-65535");
-        return -1;
-    }
-
-    if (flowinfo > 0xfffff) {
-        PyErr_SetString(PyExc_OverflowError, "flowinfo must be 0-1048575");
-        return -1;
-    }
-
-    memset(ss, 0, sizeof(struct sockaddr_storage));
-
-    if (uv_inet_pton(AF_INET, host, &addr4).code == UV_OK) {
-        /* it's an IPv4 address */
-        sa4 = (struct sockaddr_in *)ss;
-        sa4->sin_family = AF_INET;
-        sa4->sin_port = htons((short)port);
-        sa4->sin_addr = addr4;
-        return 0;
-    } else if (uv_inet_pton(AF_INET6, host, &addr6).code == UV_OK) {
-        /* it's an IPv6 address */
-        sa6 = (struct sockaddr_in6 *)ss;
-        sa6->sin6_family = AF_INET6;
-        sa6->sin6_port = htons((short)port);
-        sa6->sin6_addr = addr6;
-        sa6->sin6_flowinfo = flowinfo;
-        sa6->sin6_scope_id = scope_id;
-        return 0;
-    } else {
-        PyErr_SetString(PyExc_ValueError, "invalid IP address");
-        return -1;
-    }
-}
-
-
-/* Modified from Python Modules/socketmodule.c */
-static PyObject *
-makesockaddr(struct sockaddr *addr, int addrlen)
-{
-    static char buf[INET6_ADDRSTRLEN+1];
-    struct sockaddr_in *addr4;
-    struct sockaddr_in6 *addr6;
-
-    if (addrlen == 0) {
-        /* No address */
-        Py_RETURN_NONE;
-    }
-
-    switch (addr->sa_family) {
-    case AF_INET:
-    {
-        addr4 = (struct sockaddr_in*)addr;
-        uv_ip4_name(addr4, buf, sizeof(buf));
-        return Py_BuildValue("si", buf, ntohs(addr4->sin_port));
-    }
-
-    case AF_INET6:
-    {
-        addr6 = (struct sockaddr_in6*)addr;
-        uv_ip6_name(addr6, buf, sizeof(buf));
-        return Py_BuildValue("siII", buf, ntohs(addr6->sin6_port), ntohl(addr6->sin6_flowinfo), addr6->sin6_scope_id);
-    }
-
-    default:
-        /* If we don't know the address family, don't raise an exception -- return it as a tuple. */
-        return Py_BuildValue("is#", addr->sa_family, addr->sa_data, sizeof(addr->sa_data));
-    }
-}
-
-
-/* handle uncausht exception in a callback */
-static INLINE void
-handle_uncaught_exception(Loop *loop)
-{
-    PyObject *type, *val, *tb, *result;
-
-    ASSERT(loop);
-    ASSERT(PyErr_Occurred());
-
-    if (loop->excepthook_cb != NULL && loop->excepthook_cb != Py_None) {
-        PyErr_Fetch(&type, &val, &tb);
-        PyErr_NormalizeException(&type, &val, &tb);
-
-        if (!val) {
-            val = Py_None;
-            Py_INCREF(Py_None);
-        }
-        if (!tb) {
-            tb = Py_None;
-            Py_INCREF(Py_None);
-        }
-
-        result = PyObject_CallFunctionObjArgs(loop->excepthook_cb, type, val, tb, NULL);
-        if (result == NULL) {
-            PyErr_Print();
-        }
-        Py_XDECREF(result);
-
-        Py_DECREF(type);
-        Py_DECREF(val);
-        Py_DECREF(tb);
-
-        /* just in case*/
-        PyErr_Clear();
-    } else {
-        PyErr_Print();
-    }
-
-}
+static PyStructSequence_Desc rusage_result_desc = {
+    "rusage_result",
+    NULL,
+    rusage_result_fields,
+    16
+};
 
 
 #endif
 
-
