@@ -5,7 +5,7 @@ typedef struct {
     PyObject *callback;
     PyObject *send_handle;
     Py_buffer *views;
-    Py_buffer view[1];
+    Py_buffer viewsml[4];
     int view_count;
 } stream_write_ctx;
 
@@ -16,17 +16,9 @@ typedef struct {
     PyObject *callback;
 } stream_shutdown_ctx;
 
-static uv_buf_t
-on_stream_alloc(uv_stream_t* handle, size_t suggested_size)
-{
-    static char slab[PYUV_SLAB_SIZE];
-    UNUSED_ARG(handle);
-    return uv_buf_init(slab, sizeof(slab));
-}
-
 
 static void
-on_stream_shutdown(uv_shutdown_t* req, int status)
+pyuv__stream_shutdown_cb(uv_shutdown_t* req, int status)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
     stream_shutdown_ctx *ctx;
@@ -39,8 +31,7 @@ on_stream_shutdown(uv_shutdown_t* req, int status)
 
     if (callback != Py_None) {
         if (status < 0) {
-            uv_err_t err = uv_last_error(UV_HANDLE_LOOP(self));
-            py_errorno = PyInt_FromLong((long)err.code);
+            py_errorno = PyInt_FromLong((long)status);
         } else {
             py_errorno = Py_None;
             Py_INCREF(Py_None);
@@ -64,10 +55,10 @@ on_stream_shutdown(uv_shutdown_t* req, int status)
 
 
 static void
-on_stream_read(uv_stream_t* handle, int nread, uv_buf_t buf)
+pyuv__stream_read_cb(uv_stream_t* handle, int nread, const uv_buf_t* buf)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
-    uv_err_t err;
+    Loop *loop;
     Stream *self;
     PyObject *result, *data, *py_errorno;
     ASSERT(handle);
@@ -79,14 +70,13 @@ on_stream_read(uv_stream_t* handle, int nread, uv_buf_t buf)
     Py_INCREF(self);
 
     if (nread >= 0) {
-        data = PyBytes_FromStringAndSize(buf.base, nread);
+        data = PyBytes_FromStringAndSize(buf->base, nread);
         py_errorno = Py_None;
         Py_INCREF(Py_None);
     } else {
         data = Py_None;
         Py_INCREF(Py_None);
-        err = uv_last_error(UV_HANDLE_LOOP(self));
-        py_errorno = PyInt_FromLong((long)err.code);
+        py_errorno = PyInt_FromLong((long)nread);
         /* Stop reading, otherwise an assert blows up on unix */
         uv_read_stop(handle);
     }
@@ -99,20 +89,24 @@ on_stream_read(uv_stream_t* handle, int nread, uv_buf_t buf)
     Py_DECREF(data);
     Py_DECREF(py_errorno);
 
+    /* data has been read, unlock the buffer */
+    loop = handle->loop->data;
+    ASSERT(loop);
+    loop->buffer.in_use = False;
+
     Py_DECREF(self);
     PyGILState_Release(gstate);
 }
 
 
 static void
-on_stream_write(uv_write_t* req, int status)
+pyuv__stream_write_cb(uv_write_t* req, int status)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
     int i;
     stream_write_ctx *ctx;
     Stream *self;
     PyObject *callback, *send_handle, *result, *py_errorno;
-    uv_err_t err;
 
     ASSERT(req);
 
@@ -123,8 +117,7 @@ on_stream_write(uv_write_t* req, int status)
 
     if (callback != Py_None) {
         if (status < 0) {
-            err = uv_last_error(UV_HANDLE_LOOP(self));
-            py_errorno = PyInt_FromLong((long)err.code);
+            py_errorno = PyInt_FromLong((long)status);
         } else {
             py_errorno = Py_None;
             Py_INCREF(Py_None);
@@ -139,12 +132,11 @@ on_stream_write(uv_write_t* req, int status)
 
     Py_DECREF(callback);
     Py_XDECREF(send_handle);
-    for (i = 0; i < ctx->view_count; i++) {
+
+    for (i = 0; i < ctx->view_count; i++)
         PyBuffer_Release(&ctx->views[i]);
-    }
-    if (ctx->views != ctx->view) {
+    if (ctx->views != ctx->viewsml)
         PyMem_Free(ctx->views);
-    }
     PyMem_Free(ctx);
 
     /* Refcount was increased in the caller function */
@@ -157,7 +149,7 @@ on_stream_write(uv_write_t* req, int status)
 static PyObject *
 Stream_func_shutdown(Stream *self, PyObject *args)
 {
-    int r;
+    int err;
     stream_shutdown_ctx *ctx;
     PyObject *callback;
 
@@ -182,9 +174,9 @@ Stream_func_shutdown(Stream *self, PyObject *args)
     ctx->obj = self;
     ctx->callback = callback;
 
-    r = uv_shutdown(&ctx->req, (uv_stream_t *)UV_HANDLE(self), on_stream_shutdown);
-    if (r != 0) {
-        RAISE_STREAM_EXCEPTION(UV_HANDLE(self));
+    err = uv_shutdown(&ctx->req, (uv_stream_t *)UV_HANDLE(self), pyuv__stream_shutdown_cb);
+    if (err < 0) {
+        RAISE_STREAM_EXCEPTION(err, UV_HANDLE(self));
         goto error;
     }
 
@@ -203,7 +195,7 @@ error:
 static PyObject *
 Stream_func_start_read(Stream *self, PyObject *args)
 {
-    int r;
+    int err;
     PyObject *tmp, *callback;
 
     tmp = NULL;
@@ -220,9 +212,9 @@ Stream_func_start_read(Stream *self, PyObject *args)
         return NULL;
     }
 
-    r = uv_read_start((uv_stream_t *)UV_HANDLE(self), (uv_alloc_cb)on_stream_alloc, (uv_read_cb)on_stream_read);
-    if (r != 0) {
-        RAISE_STREAM_EXCEPTION(UV_HANDLE(self));
+    err = uv_read_start((uv_stream_t *)UV_HANDLE(self), (uv_alloc_cb)pyuv__alloc_cb, (uv_read_cb)pyuv__stream_read_cb);
+    if (err < 0) {
+        RAISE_STREAM_EXCEPTION(err, UV_HANDLE(self));
         return NULL;
     }
 
@@ -231,6 +223,8 @@ Stream_func_start_read(Stream *self, PyObject *args)
     self->on_read_cb = callback;
     Py_XDECREF(tmp);
 
+    PYUV_HANDLE_INCREF(self);
+
     Py_RETURN_NONE;
 }
 
@@ -238,46 +232,180 @@ Stream_func_start_read(Stream *self, PyObject *args)
 static PyObject *
 Stream_func_stop_read(Stream *self)
 {
-    int r;
+    int err;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    r = uv_read_stop((uv_stream_t *)UV_HANDLE(self));
-    if (r != 0) {
-        RAISE_STREAM_EXCEPTION(UV_HANDLE(self));
+    err = uv_read_stop((uv_stream_t *)UV_HANDLE(self));
+    if (err < 0) {
+        RAISE_STREAM_EXCEPTION(err, UV_HANDLE(self));
         return NULL;
     }
 
     Py_XDECREF(self->on_read_cb);
     self->on_read_cb = NULL;
 
+    PYUV_HANDLE_DECREF(self);
+
     Py_RETURN_NONE;
 }
 
 
-static INLINE PyObject *
-pyuv_stream_write(Stream *self, stream_write_ctx *ctx, Py_buffer *views, uv_buf_t *bufs, int buf_count, PyObject *callback, PyObject *send_handle)
+static PyObject *
+Stream_func_try_write(Stream *self, PyObject *args)
 {
-    int i, r;
+    int err;
+    uv_buf_t buf;
+    Py_buffer view;
+
+    RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
+    RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
+
+    if (!PyArg_ParseTuple(args, PYUV_BYTES"*:try_write", &view)) {
+        return NULL;
+    }
+
+    buf = uv_buf_init(view.buf, view.len);
+    err = uv_try_write((uv_stream_t *)UV_HANDLE(self), &buf, 1);
+    if (err < 0) {
+        RAISE_STREAM_EXCEPTION(err, UV_HANDLE(self));
+        PyBuffer_Release(&view);
+        return NULL;
+    }
+
+    PyBuffer_Release(&view);
+    return PyInt_FromLong((long)err);
+}
+
+
+static PyObject *
+pyuv__stream_write_bytes(Stream *self, PyObject *data, PyObject *callback, PyObject *send_handle)
+{
+    int err;
+    uv_buf_t buf;
+    stream_write_ctx *ctx;
+    Py_buffer *view;
+
+    ctx = PyMem_Malloc(sizeof *ctx);
+    if (!ctx) {
+        PyErr_NoMemory();
+        return NULL;
+    }
+
+    ctx->views = ctx->viewsml;
+    view = &ctx->views[0];
+
+    if (PyObject_GetBuffer(data, view, PyBUF_SIMPLE) != 0) {
+        PyMem_Free(ctx);
+        return NULL;
+    }
+
+    ctx->view_count = 1;
+    ctx->obj = self;
+    ctx->callback = callback;
+    ctx->send_handle = send_handle;
 
     Py_INCREF(callback);
     Py_XINCREF(send_handle);
 
-    ctx->obj = self;
-    ctx->callback = callback;
-    ctx->send_handle = send_handle;
-    ctx->views = views;
-    ctx->view_count = buf_count;
-
-    if (send_handle) {
-        r = uv_write2(&ctx->req, (uv_stream_t *)UV_HANDLE(self), bufs, buf_count, (uv_stream_t *)UV_HANDLE(send_handle), on_stream_write);
+    buf = uv_buf_init(view->buf, view->len);
+    if (send_handle != NULL) {
+        ASSERT(UV_HANDLE(self)->type == UV_NAMED_PIPE);
+        err = uv_write2(&ctx->req, (uv_stream_t *)UV_HANDLE(self), &buf, 1, (uv_stream_t *)UV_HANDLE(send_handle), pyuv__stream_write_cb);
     } else {
-        r = uv_write(&ctx->req, (uv_stream_t *)UV_HANDLE(self), bufs, buf_count, on_stream_write);
+        err = uv_write(&ctx->req, (uv_stream_t *)UV_HANDLE(self), &buf, 1, pyuv__stream_write_cb);
     }
 
-    if (r != 0) {
-        RAISE_STREAM_EXCEPTION(UV_HANDLE(self));
+    if (err < 0) {
+        RAISE_STREAM_EXCEPTION(err, UV_HANDLE(self));
+        Py_DECREF(callback);
+        Py_XDECREF(send_handle);
+        PyBuffer_Release(view);
+        PyMem_Free(ctx);
+        return NULL;
+    }
+
+    /* Increase refcount so that object is not removed before the callback is called */
+    Py_INCREF(self);
+
+    Py_RETURN_NONE;
+}
+
+
+static PyObject *
+pyuv__stream_write_sequence(Stream *self, PyObject *data, PyObject *callback, PyObject *send_handle)
+{
+    int err;
+    stream_write_ctx *ctx;
+    PyObject *data_fast, *item;
+    Py_ssize_t i, j, buf_count;
+
+    data_fast = PySequence_Fast(data, "data must be an iterable");
+    if (data_fast == NULL)
+        return NULL;
+
+    buf_count = PySequence_Fast_GET_SIZE(data_fast);
+    if (buf_count > INT_MAX) {
+        PyErr_SetString(PyExc_ValueError, "iterable is too long");
+        Py_DECREF(data_fast);
+        return NULL;
+    }
+
+    if (buf_count == 0) {
+        PyErr_SetString(PyExc_ValueError, "iterable is empty");
+        Py_DECREF(data_fast);
+        return NULL;
+    }
+
+    ctx = PyMem_Malloc(sizeof *ctx);
+    if (!ctx) {
+        PyErr_NoMemory();
+        Py_DECREF(data_fast);
+        return NULL;
+    }
+
+    ctx->views = ctx->viewsml;
+    if (buf_count > ARRAY_SIZE(ctx->viewsml))
+        ctx->views = PyMem_Malloc(sizeof(Py_buffer) * buf_count);
+    if (!ctx->views) {
+        PyErr_NoMemory();
+        PyMem_Free(ctx);
+        Py_DECREF(data_fast);
+        return NULL;
+    }
+    ctx->view_count = buf_count;
+
+    {
+        STACK_ARRAY(uv_buf_t, bufs, buf_count);
+
+        for (i = 0; i < buf_count; i++) {
+            item = PySequence_Fast_GET_ITEM(data_fast, i);
+            if (PyObject_GetBuffer(item, &ctx->views[i], PyBUF_SIMPLE) != 0)
+                goto error;
+            bufs[i].base = ctx->views[i].buf;
+            bufs[i].len = ctx->views[i].len;
+        }
+
+        ctx->obj = self;
+        ctx->callback = callback;
+        ctx->send_handle = send_handle;
+
+        Py_INCREF(callback);
+        Py_XINCREF(send_handle);
+
+        if (send_handle != NULL) {
+            ASSERT(UV_HANDLE(self)->type == UV_NAMED_PIPE);
+            err = uv_write2(&ctx->req, (uv_stream_t *)UV_HANDLE(self), bufs, buf_count, (uv_stream_t *)UV_HANDLE(send_handle), pyuv__stream_write_cb);
+        } else {
+            err = uv_write(&ctx->req, (uv_stream_t *)UV_HANDLE(self), bufs, buf_count, pyuv__stream_write_cb);
+        }
+    }
+
+    if (err < 0) {
+        RAISE_STREAM_EXCEPTION(err, UV_HANDLE(self));
+        Py_DECREF(callback);
+        Py_XDECREF(send_handle);
         goto error;
     }
 
@@ -287,15 +415,12 @@ pyuv_stream_write(Stream *self, stream_write_ctx *ctx, Py_buffer *views, uv_buf_
     Py_RETURN_NONE;
 
 error:
-    Py_DECREF(callback);
-    Py_XDECREF(send_handle);
-    for (i = 0; i < buf_count; i++) {
-        PyBuffer_Release(&views[i]);
-    }
-    if (ctx->views != ctx->view) {
-        PyMem_Free(views);
-    }
+    for (j = 0; j < i; j++)
+        PyBuffer_Release(&ctx->views[j]);
+    if (ctx->views != ctx->viewsml)
+        PyMem_Free(ctx->views);
     PyMem_Free(ctx);
+    Py_XDECREF(data_fast);
     return NULL;
 }
 
@@ -303,82 +428,51 @@ error:
 static PyObject *
 Stream_func_write(Stream *self, PyObject *args)
 {
-    uv_buf_t buf;
-    stream_write_ctx *ctx;
-    Py_buffer *view;
+    PyObject *data;
     PyObject *callback = Py_None;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    ctx = PyMem_Malloc(sizeof *ctx);
-    if (!ctx) {
-        PyErr_NoMemory();
-        return NULL;
-    }
-
-    view = &ctx->view[0];
-
-    if (!PyArg_ParseTuple(args, PYUV_BYTES"*|O:write", view, &callback)) {
-        PyMem_Free(ctx);
+    if (!PyArg_ParseTuple(args, "O|O:write", &data, &callback)) {
         return NULL;
     }
 
     if (callback != Py_None && !PyCallable_Check(callback)) {
-        PyBuffer_Release(view);
-        PyMem_Free(ctx);
-        PyErr_SetString(PyExc_TypeError, "a callable or None is required");
+        PyErr_SetString(PyExc_TypeError, "'callback' must be a callable or None");
         return NULL;
     }
 
-    buf = uv_buf_init(view->buf, view->len);
-
-    return pyuv_stream_write(self, ctx, view, &buf, 1, callback, NULL);
+    if (PyObject_CheckBuffer(data)) {
+        return pyuv__stream_write_bytes(self, data, callback, NULL);
+    } else if (!PyUnicode_Check(data) && PySequence_Check(data)) {
+        return pyuv__stream_write_sequence(self, data, callback, NULL);
+    } else {
+        PyErr_SetString(PyExc_TypeError, "only bytes and sequences are supported");
+        return NULL;
+    }
 }
 
 
 static PyObject *
-Stream_func_writelines(Stream *self, PyObject *args)
+Stream_func_fileno(Stream *self)
 {
-    int r, buf_count;
-    Py_buffer *views;
-    PyObject *callback, *seq, *ret;
-    uv_buf_t *bufs;
-    stream_write_ctx *ctx;
-
-    callback = Py_None;
+    int err;
+    uv_os_fd_t fd;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    if (!PyArg_ParseTuple(args, "O|O:writelines", &seq, &callback)) {
+    err = uv_fileno(UV_HANDLE(self), &fd);
+    if (err < 0) {
+        RAISE_STREAM_EXCEPTION(err, UV_HANDLE(self));
         return NULL;
     }
 
-    if (callback != Py_None && !PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable or None is required");
-        return NULL;
-    }
-
-    ctx = PyMem_Malloc(sizeof *ctx);
-    if (!ctx) {
-        PyErr_NoMemory();
-        return NULL;
-    }
-
-    r = pyseq2uvbuf(seq, &views, &bufs, &buf_count);
-    if (r != 0) {
-        /* error is already set */
-        PyMem_Free(ctx);
-        return NULL;
-    }
-
-    ret = pyuv_stream_write(self, ctx, views, bufs, buf_count, callback, NULL);
-
-    /* uv_write copies the uv_buf_t structures, so we can free them now */
-    PyMem_Free(bufs);
-
-    return ret;
+    /* us_os_fd_t is a HANDLE on Windows which is a 64-bit data type but which
+     * is guaranteed to contain only values < 2^24.
+     * For more information, see: http://www.viva64.com/en/k/0005/ */
+    return PyInt_FromLong((long) fd);
 }
 
 
@@ -445,10 +539,11 @@ Stream_tp_clear(Stream *self)
 static PyMethodDef
 Stream_tp_methods[] = {
     { "shutdown", (PyCFunction)Stream_func_shutdown, METH_VARARGS, "Shutdown the write side of this Stream." },
+    { "try_write", (PyCFunction)Stream_func_try_write, METH_VARARGS, "Try to write data on the stream." },
     { "write", (PyCFunction)Stream_func_write, METH_VARARGS, "Write data on the stream." },
-    { "writelines", (PyCFunction)Stream_func_writelines, METH_VARARGS, "Write a sequence of data on the stream." },
     { "start_read", (PyCFunction)Stream_func_start_read, METH_VARARGS, "Start read data from the connected endpoint." },
     { "stop_read", (PyCFunction)Stream_func_stop_read, METH_NOARGS, "Stop read data from the connected endpoint." },
+    { "fileno", (PyCFunction)Stream_func_fileno, METH_NOARGS, "Returns the libuv OS handle." },
     { NULL }
 };
 
