@@ -2,12 +2,15 @@
 import os
 import shutil
 import stat
+import unittest
 
-from common import unittest2
+from common import TestCase
 import pyuv
 
+
 # Make stat return integers
 os.stat_float_times(False)
+pyuv.fs.stat_float_times(False)
 
 
 BAD_FILE = 'test_file_bad'
@@ -18,20 +21,44 @@ TEST_DIR = 'test-dir'
 TEST_DIR2 = 'test-dir_2'
 BAD_DIR = 'test-dir-bad'
 MAX_INT32_VALUE = 2 ** 31 - 1
+OFFSET_VALUE = MAX_INT32_VALUE if not os.name == 'nt' else 2 ** 8 - 1
 
 
-class FSTestStat(unittest2.TestCase):
+class FileTestCase(TestCase):
+
+    TEST_FILE_CONTENT = 'test'
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(FileTestCase, self).setUp()
         with open(TEST_FILE, 'w') as f:
-            f.write('test')
+            f.write(self.TEST_FILE_CONTENT)
 
     def tearDown(self):
-        os.remove(TEST_FILE)
+        try:
+            os.remove(TEST_FILE)
+        except OSError:
+            pass
+        super(FileTestCase, self).tearDown()
 
-    def stat_cb(self, loop, path, stat_result, errorno):
-        self.errorno = errorno
+
+class FSTestRequestDict(FileTestCase):
+
+    def stat_cb(self, req):
+        self.errorno = req.error
+        self.assertEqual(req.test, 'test123')
+
+    def test_request_dict(self):
+        self.errorno = None
+        req = pyuv.fs.stat(self.loop, TEST_FILE, self.stat_cb)
+        req.test = 'test123'
+        self.loop.run()
+        self.assertEqual(self.errorno, None)
+
+
+class FSTestStat(FileTestCase):
+
+    def stat_cb(self, req):
+        self.errorno = req.error
 
     def test_stat_error(self):
         self.errorno = None
@@ -59,27 +86,25 @@ class FSTestStat(unittest2.TestCase):
         self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)
 
 
-class FSTestLstat(unittest2.TestCase):
+class FSTestLstat(FileTestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write('test')
+        super(FSTestLstat, self).setUp()
         try:
             pyuv.fs.symlink(self.loop, TEST_FILE, TEST_LINK, 0)
         except pyuv.error.FSError as e:
             if e.args[0] == pyuv.errno.UV_EPERM:
-                raise unittest2.SkipTest("Symlinks not permitted")
+                raise unittest.SkipTest("Symlinks not permitted")
 
     def tearDown(self):
-        os.remove(TEST_FILE)
         try:
             os.remove(TEST_LINK)
         except OSError:
             pass
+        super(FSTestLstat, self).tearDown()
 
-    def stat_cb(self, loop, path, stat_result, errorno):
-        self.errorno = errorno
+    def stat_cb(self, req):
+        self.errorno = req.error
 
     def test_lstat(self):
         self.errorno = None
@@ -88,18 +113,10 @@ class FSTestLstat(unittest2.TestCase):
         self.assertEqual(self.errorno, None)
 
 
-class FSTestFstat(unittest2.TestCase):
+class FSTestFstat(FileTestCase):
 
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write('test')
-
-    def tearDown(self):
-        os.remove(TEST_FILE)
-
-    def fstat_cb(self, loop, path, stat_data, errorno):
-        self.errorno = errorno
+    def fstat_cb(self, req):
+        self.errorno = req.error
 
     def test_fstat(self):
         self.errorno = None
@@ -116,21 +133,10 @@ class FSTestFstat(unittest2.TestCase):
         self.assertNotEqual(self.stat_data, None)
 
 
-class FSTestUnlink(unittest2.TestCase):
+class FSTestUnlink(FileTestCase):
 
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write('test')
-
-    def tearDown(self):
-        try:
-            os.remove(TEST_FILE)
-        except OSError:
-            pass
-
-    def bad_unlink_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def bad_unlink_cb(self, req):
+        self.errorno = req.error
 
     def test_bad_unlink(self):
         self.errorno = None
@@ -138,8 +144,8 @@ class FSTestUnlink(unittest2.TestCase):
         self.loop.run()
         self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)
 
-    def unlink_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def unlink_cb(self, req):
+        self.errorno = req.error
 
     def test_unlink(self):
         self.errorno = None
@@ -160,10 +166,10 @@ class FSTestUnlink(unittest2.TestCase):
         self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)
 
 
-class FSTestMkdir(unittest2.TestCase):
+class FSTestMkdir(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(FSTestMkdir, self).setUp()
         os.mkdir(BAD_DIR, 0o755)
 
     def tearDown(self):
@@ -172,9 +178,10 @@ class FSTestMkdir(unittest2.TestCase):
             os.rmdir(TEST_DIR)
         except OSError:
             pass
+        super(FSTestMkdir, self).tearDown()
 
-    def mkdir_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def mkdir_cb(self, req):
+        self.errorno = req.error
 
     def test_bad_mkdir(self):
         self.errorno = None
@@ -203,10 +210,10 @@ class FSTestMkdir(unittest2.TestCase):
         self.assertEqual(self.errorno, pyuv.errno.UV_EEXIST)
 
 
-class FSTestRmdir(unittest2.TestCase):
+class FSTestRmdir(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(FSTestRmdir, self).setUp()
         os.mkdir(TEST_DIR, 0o755)
 
     def tearDown(self):
@@ -214,9 +221,10 @@ class FSTestRmdir(unittest2.TestCase):
             os.rmdir(TEST_DIR)
         except OSError:
             pass
+        super(FSTestRmdir, self).tearDown()
 
-    def rmdir_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def rmdir_cb(self, req):
+        self.errorno = req.error
 
     def test_bad_rmdir(self):
         self.errorno = None
@@ -245,25 +253,17 @@ class FSTestRmdir(unittest2.TestCase):
         self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)
 
 
-class FSTestRename(unittest2.TestCase):
-
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write('test')
+class FSTestRename(FileTestCase):
 
     def tearDown(self):
-        try:
-            os.remove(TEST_FILE)
-        except OSError:
-            pass
         try:
             os.remove(TEST_FILE2)
         except OSError:
             pass
+        super(FSTestRename, self).tearDown()
 
-    def rename_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def rename_cb(self, req):
+        self.errorno = req.error
 
     def test_rename(self):
         self.errorno = None
@@ -279,18 +279,10 @@ class FSTestRename(unittest2.TestCase):
         self.assertTrue(os.path.exists(TEST_FILE2))
 
 
-class FSTestChmod(unittest2.TestCase):
+class FSTestChmod(FileTestCase):
 
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write('test')
-
-    def tearDown(self):
-        os.remove(TEST_FILE)
-
-    def chmod_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def chmod_cb(self, req):
+        self.errorno = req.error
 
     def test_chmod(self):
         self.errorno = None
@@ -306,18 +298,10 @@ class FSTestChmod(unittest2.TestCase):
         self.assertTrue(bool(mode & stat.S_IRWXU) and bool(mode & stat.S_IRWXG) and bool(mode & stat.S_IRWXO))
 
 
-class FSTestFchmod(unittest2.TestCase):
+class FSTestFchmod(FileTestCase):
 
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write('test')
-
-    def tearDown(self):
-        os.remove(TEST_FILE)
-
-    def fchmod_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def fchmod_cb(self, req):
+        self.errorno = req.error
 
     def test_fchmod(self):
         self.errorno = None
@@ -337,19 +321,14 @@ class FSTestFchmod(unittest2.TestCase):
         self.assertTrue(bool(mode & stat.S_IRWXU) and bool(mode & stat.S_IRWXG) and bool(mode & stat.S_IRWXO))
 
 
-class FSTestLink(unittest2.TestCase):
-
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write('test')
+class FSTestLink(FileTestCase):
 
     def tearDown(self):
-        os.remove(TEST_FILE)
         os.remove(TEST_LINK)
+        super(FSTestLink, self).tearDown()
 
-    def link_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def link_cb(self, req):
+        self.errorno = req.error
 
     def test_link(self):
         self.errorno = None
@@ -363,29 +342,24 @@ class FSTestLink(unittest2.TestCase):
         self.assertEqual(os.stat(TEST_FILE).st_ino, os.stat(TEST_LINK).st_ino)
 
 
-class FSTestSymlink(unittest2.TestCase):
-
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write('test')
+class FSTestSymlink(FileTestCase):
 
     def tearDown(self):
-        os.remove(TEST_FILE)
         try:
             os.remove(TEST_LINK)
         except OSError:
             pass
+        super(FSTestSymlink, self).tearDown()
 
-    def symlink_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def symlink_cb(self, req):
+        self.errorno = req.error
 
     def test_symlink(self):
         self.errorno = None
         pyuv.fs.symlink(self.loop, TEST_FILE, TEST_LINK, 0, self.symlink_cb)
         self.loop.run()
         if self.errorno == pyuv.errno.UV_EPERM:
-            raise unittest2.SkipTest("Symlinks not permitted")
+            raise unittest.SkipTest("Symlinks not permitted")
         self.assertEqual(self.errorno, None)
         self.assertTrue(os.stat(TEST_LINK).st_mode & stat.S_IFLNK)
 
@@ -394,32 +368,30 @@ class FSTestSymlink(unittest2.TestCase):
             pyuv.fs.symlink(self.loop, TEST_FILE, TEST_LINK, 0)
         except pyuv.error.FSError as e:
             if e.args[0] == pyuv.errno.UV_EPERM:
-                raise unittest2.SkipTest("Symlinks not permitted")
+                raise unittest.SkipTest("Symlinks not permitted")
         self.assertTrue(os.stat(TEST_LINK).st_mode & stat.S_IFLNK)
 
 
-class FSTestReadlink(unittest2.TestCase):
+class FSTestReadlink(FileTestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write('test')
+        super(FSTestReadlink, self).setUp()
         try:
             pyuv.fs.symlink(self.loop, TEST_FILE, TEST_LINK, 0)
         except pyuv.error.FSError as e:
             if e.args[0] == pyuv.errno.UV_EPERM:
-                raise unittest2.SkipTest("Symlinks not permitted")
+                raise unittest.SkipTest("Symlinks not permitted")
 
     def tearDown(self):
-        os.remove(TEST_FILE)
         try:
             os.remove(TEST_LINK)
         except OSError:
             pass
+        super(FSTestReadlink, self).tearDown()
 
-    def readlink_cb(self, loop, path, errorno):
-        self.errorno = errorno
-        self.link_path = path
+    def readlink_cb(self, req):
+        self.errorno = req.error
+        self.link_path = req.result
 
     def test_readlink(self):
         self.errorno = None
@@ -434,18 +406,10 @@ class FSTestReadlink(unittest2.TestCase):
         self.assertEqual(self.link_path, TEST_FILE)
 
 
-class FSTestChown(unittest2.TestCase):
+class FSTestChown(FileTestCase):
 
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write('test')
-
-    def tearDown(self):
-        os.remove(TEST_FILE)
-
-    def chown_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def chown_cb(self, req):
+        self.errorno = req.error
 
     def test_chown(self):
         self.errorno = None
@@ -457,18 +421,10 @@ class FSTestChown(unittest2.TestCase):
         pyuv.fs.chown(self.loop, TEST_FILE, -1, -1)
 
 
-class FSTestFchown(unittest2.TestCase):
+class FSTestFchown(FileTestCase):
 
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write('test')
-
-    def tearDown(self):
-        os.remove(TEST_FILE)
-
-    def fchown_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def fchown_cb(self, req):
+        self.errorno = req.error
 
     def test_fchown(self):
         self.errorno = None
@@ -484,10 +440,10 @@ class FSTestFchown(unittest2.TestCase):
         pyuv.fs.close(self.loop, fd)
 
 
-class FSTestOpen(unittest2.TestCase):
+class FSTestOpen(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(FSTestOpen, self).setUp()
         try:
             os.remove(TEST_FILE)
         except OSError:
@@ -498,13 +454,15 @@ class FSTestOpen(unittest2.TestCase):
             os.remove(TEST_FILE)
         except OSError:
             pass
+        super(FSTestOpen, self).tearDown()
 
-    def close_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def close_cb(self, req):
+        self.errorno = req.error
 
-    def open_cb(self, loop, path, fd, errorno):
+    def open_cb(self, req):
+        fd = req.result
         self.assertNotEqual(fd, None)
-        self.assertEqual(errorno, None)
+        self.assertEqual(req.error, None)
         pyuv.fs.close(self.loop, fd, self.close_cb)
 
     def test_open_create(self):
@@ -513,9 +471,9 @@ class FSTestOpen(unittest2.TestCase):
         self.loop.run()
         self.assertEqual(self.errorno, None)
 
-    def open_noent_cb(self, loop, path, fd, errorno):
-        self.fd = fd
-        self.errorno = errorno
+    def open_noent_cb(self, req):
+        self.fd = req.result
+        self.errorno = req.error
 
     def test_open_noent(self):
         self.fd = None
@@ -539,19 +497,13 @@ class FSTestOpen(unittest2.TestCase):
         self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)
 
 
-class FSTestRead(unittest2.TestCase):
+class FSTestRead(FileTestCase):
 
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write('test1234567890')
+    TEST_FILE_CONTENT = 'test1234567890'
 
-    def tearDown(self):
-        os.remove(TEST_FILE)
-
-    def read_cb(self, loop, path, read_data, errorno):
-        self.errorno = errorno
-        self.data = read_data
+    def read_cb(self, req):
+        self.errorno = req.error
+        self.data = req.result
         pyuv.fs.close(self.loop, self.fd)
 
     def test_read(self):
@@ -571,32 +523,33 @@ class FSTestRead(unittest2.TestCase):
 
     def test_read_offset(self):
         with open(TEST_FILE, 'w') as f:
-            f.seek(MAX_INT32_VALUE)
+            f.seek(OFFSET_VALUE)
             f.write('test1234567890')
         fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDONLY, stat.S_IREAD)
-        data = pyuv.fs.read(self.loop, fd, 4, MAX_INT32_VALUE + 4)
+        data = pyuv.fs.read(self.loop, fd, 4, OFFSET_VALUE + 4)
         pyuv.fs.close(self.loop, fd)
         self.assertEqual(data, b'1234')
 
 
-class FSTestWrite(unittest2.TestCase):
+class FSTestWrite(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(FSTestWrite, self).setUp()
         self.fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_WRONLY|os.O_CREAT|os.O_TRUNC, stat.S_IWRITE|stat.S_IREAD)
 
     def tearDown(self):
         os.remove(TEST_FILE)
+        super(FSTestWrite, self).tearDown()
 
-    def write_cb(self, loop, path, bytes_written, errorno):
+    def write_cb(self, req):
         pyuv.fs.close(self.loop, self.fd)
-        self.bytes_written = bytes_written
-        self.errorno = errorno
+        self.bytes_written = req.result
+        self.errorno = req.error
 
     def test_write(self):
         self.bytes_written = None
         self.errorno = None
-        pyuv.fs.write(self.loop, self.fd, "TEST", -1, self.write_cb)
+        pyuv.fs.write(self.loop, self.fd, b"TEST", -1, self.write_cb)
         self.loop.run()
         self.assertEqual(self.bytes_written, 4)
         self.assertEqual(self.errorno, None)
@@ -606,7 +559,7 @@ class FSTestWrite(unittest2.TestCase):
     def test_write_null(self):
         self.bytes_written = None
         self.errorno = None
-        pyuv.fs.write(self.loop, self.fd, "TES\x00T", -1, self.write_cb)
+        pyuv.fs.write(self.loop, self.fd, b"TES\x00T", -1, self.write_cb)
         self.loop.run()
         self.assertEqual(self.bytes_written, 5)
         self.assertEqual(self.errorno, None)
@@ -614,41 +567,38 @@ class FSTestWrite(unittest2.TestCase):
             self.assertEqual(fobj.read(), "TES\x00T")
 
     def test_write_sync(self):
-        self.bytes_written = pyuv.fs.write(self.loop, self.fd, "TEST", -1)
+        self.bytes_written = pyuv.fs.write(self.loop, self.fd, b"TEST", -1)
         pyuv.fs.close(self.loop, self.fd)
         self.assertEqual(self.bytes_written, 4)
         with open(TEST_FILE, 'r') as fobj:
             self.assertEqual(fobj.read(), "TEST")
 
     def test_write_offset(self):
-        offset = MAX_INT32_VALUE + 4
-        self.bytes_written = pyuv.fs.write(self.loop, self.fd, "TEST", offset)
+        offset = OFFSET_VALUE + 4
+        self.bytes_written = pyuv.fs.write(self.loop, self.fd, b"TEST", offset)
         pyuv.fs.close(self.loop, self.fd)
         with open(TEST_FILE, 'r') as fobj:
             fobj.seek(offset)
             self.assertEqual(fobj.read(), "TEST")
 
 
-class FSTestFsync(unittest2.TestCase):
+class FSTestFsync(TestCase):
 
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-
-    def write_cb(self, loop, path, bytes_written, errorno):
-        self.assertEqual(bytes_written, 4)
-        self.assertEqual(errorno, None)
+    def write_cb(self, req):
+        self.assertEqual(req.result, 4)
+        self.assertEqual(req.error, None)
         pyuv.fs.fdatasync(self.loop, self.fd, self.fdatasync_cb)
 
-    def fdatasync_cb(self, loop, path, errorno):
-        self.assertEqual(errorno, None)
+    def fdatasync_cb(self, req):
+        self.assertEqual(req.error, None)
         pyuv.fs.fsync(self.loop, self.fd, self.fsync_cb)
 
-    def fsync_cb(self, loop, path, errorno):
-        self.assertEqual(errorno, None)
+    def fsync_cb(self, req):
+        self.assertEqual(req.error, None)
 
     def test_fsync(self):
         self.fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR|os.O_CREAT|os.O_TRUNC, stat.S_IREAD|stat.S_IWRITE)
-        pyuv.fs.write(self.loop, self.fd, "TEST", -1, self.write_cb)
+        pyuv.fs.write(self.loop, self.fd, b"TEST", -1, self.write_cb)
         self.loop.run()
         pyuv.fs.close(self.loop, self.fd)
         with open(TEST_FILE, 'r') as fobj:
@@ -656,7 +606,7 @@ class FSTestFsync(unittest2.TestCase):
 
     def test_fsync_sync(self):
         self.fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR|os.O_CREAT|os.O_TRUNC, stat.S_IREAD|stat.S_IWRITE)
-        pyuv.fs.write(self.loop, self.fd, "TEST", -1)
+        pyuv.fs.write(self.loop, self.fd, b"TEST", -1)
         pyuv.fs.fdatasync(self.loop, self.fd)
         pyuv.fs.fsync(self.loop, self.fd)
         pyuv.fs.close(self.loop, self.fd)
@@ -664,18 +614,12 @@ class FSTestFsync(unittest2.TestCase):
             self.assertEqual(fobj.read(), "TEST")
 
 
-class FSTestFtruncate(unittest2.TestCase):
+class FSTestFtruncate(FileTestCase):
 
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write("test-data")
+    TEST_FILE_CONTENT = "test-data"
 
-    def tearDown(self):
-        os.remove(TEST_FILE)
-
-    def ftruncate_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def ftruncate_cb(self, req):
+        self.errorno = req.error
         pyuv.fs.close(self.loop, self.fd)
 
     def test_ftruncate1(self):
@@ -704,10 +648,10 @@ class FSTestFtruncate(unittest2.TestCase):
             self.assertEqual(fobj.read(), "")
 
 
-class FSTestReaddir(unittest2.TestCase):
+class FSTestScandir(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(FSTestScandir, self).setUp()
         os.mkdir(TEST_DIR, 0o755)
         os.mkdir(os.path.join(TEST_DIR, TEST_DIR2), 0o755)
         with open(os.path.join(TEST_DIR, TEST_FILE), 'w') as f:
@@ -717,38 +661,39 @@ class FSTestReaddir(unittest2.TestCase):
 
     def tearDown(self):
         shutil.rmtree(TEST_DIR)
+        super(FSTestScandir, self).tearDown()
 
-    def readdir_cb(self, loop, path, files, errorno):
-        self.errorno = errorno
-        self.files = files
+    def scandir_cb(self, req):
+        self.errorno = req.error
+        self.files = req.result
 
-    def test_bad_readdir(self):
+    def test_bad_scandir(self):
         self.errorno = None
         self.files = None
-        pyuv.fs.readdir(self.loop, BAD_DIR, 0, self.readdir_cb)
+        pyuv.fs.scandir(self.loop, BAD_DIR, 0, self.scandir_cb)
         self.loop.run()
         self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)
 
-    def test_readdir(self):
+    def test_scandir(self):
         self.errorno = None
         self.files = None
-        pyuv.fs.readdir(self.loop, TEST_DIR, 0, self.readdir_cb)
+        pyuv.fs.scandir(self.loop, TEST_DIR, 0, self.scandir_cb)
         self.loop.run()
         self.assertEqual(self.errorno, None)
         self.assertTrue(TEST_FILE in self.files)
         self.assertTrue(TEST_FILE2 in self.files)
         self.assertTrue(TEST_DIR2 in self.files)
 
-    def test_readdir_sync(self):
-        self.files = pyuv.fs.readdir(self.loop, TEST_DIR, 0)
+    def test_scandir_sync(self):
+        self.files = pyuv.fs.scandir(self.loop, TEST_DIR, 0)
         self.assertNotEqual(self.files, None)
         self.assertTrue(TEST_FILE in self.files)
         self.assertTrue(TEST_FILE2 in self.files)
         self.assertTrue(TEST_DIR2 in self.files)
 
-    def test_readdir_error_sync(self):
+    def test_scandir_error_sync(self):
         try:
-            pyuv.fs.readdir(self.loop, BAD_DIR, 0)
+            pyuv.fs.scandir(self.loop, BAD_DIR, 0)
         except pyuv.error.FSError as e:
             self.errorno = e.args[0]
         else:
@@ -756,10 +701,10 @@ class FSTestReaddir(unittest2.TestCase):
         self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)
 
 
-class FSTestSendfile(unittest2.TestCase):
+class FSTestSendfile(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(FSTestSendfile, self).setUp()
         with open(TEST_FILE, 'w') as f:
             f.write("begin\n")
             os.lseek(f.fileno(), 65536, os.SEEK_CUR)
@@ -769,10 +714,11 @@ class FSTestSendfile(unittest2.TestCase):
     def tearDown(self):
         os.remove(TEST_FILE)
         os.remove(TEST_FILE2)
+        super(FSTestSendfile, self).tearDown()
 
-    def sendfile_cb(self, loop, path, bytes_written, errorno):
-        self.bytes_written = bytes_written
-        self.errorno = errorno
+    def sendfile_cb(self, req):
+        self.bytes_written = req.result
+        self.errorno = req.error
 
     def test_sendfile(self):
         self.result = None
@@ -799,7 +745,7 @@ class FSTestSendfile(unittest2.TestCase):
                 self.assertEqual(fobj1.read(), fobj2.read())
 
     def test_sendfile_offset(self):
-        offset = MAX_INT32_VALUE + 1
+        offset = OFFSET_VALUE + 1
         with open(TEST_FILE, 'w') as f:
             f.seek(offset)
             f.write("test")
@@ -815,19 +761,14 @@ class FSTestSendfile(unittest2.TestCase):
                 self.assertEqual(fobj1.read(), fobj2.read())
 
 
-class FSTestUtime(unittest2.TestCase):
+class FSTestUtime(FileTestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        with open(TEST_FILE, 'w') as f:
-            f.write("test")
+        super(FSTestUtime, self).setUp()
         self.fd = None
 
-    def tearDown(self):
-        os.remove(TEST_FILE)
-
-    def utime_cb(self, loop, path, errorno):
-        self.errorno = errorno
+    def utime_cb(self, req):
+        self.errorno = req.error
         if self.fd is not None:
             pyuv.fs.close(self.loop, self.fd)
 
@@ -868,10 +809,10 @@ class FSTestUtime(unittest2.TestCase):
         self.assertEqual(s.st_mtime, mtime)
 
 
-class FSEventTestBasic(unittest2.TestCase):
+class FSTestAccess(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(FSTestAccess, self).setUp()
         with open(TEST_FILE, 'w') as f:
             f.write("test")
 
@@ -880,12 +821,47 @@ class FSEventTestBasic(unittest2.TestCase):
             os.remove(TEST_FILE)
         except OSError:
             pass
+        super(FSTestAccess, self).tearDown()
+
+    def access_cb(self, req):
+        self.errorno = req.error
+
+    def test_bad_access(self):
+        self.errorno = None
+        pyuv.fs.access(self.loop, BAD_FILE, os.F_OK, self.access_cb)
+        self.loop.run()
+        self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)
+
+    def test_access(self):
+        self.errorno = None
+        pyuv.fs.access(self.loop, TEST_FILE, os.F_OK, self.access_cb)
+        self.loop.run()
+        self.assertEqual(self.errorno, None)
+
+    def test_access_sync(self):
+        pyuv.fs.access(self.loop, TEST_FILE, os.F_OK)
+
+    def test_access_error_sync(self):
+        try:
+            pyuv.fs.access(self.loop, BAD_FILE, os.F_OK)
+        except pyuv.error.FSError as e:
+            self.errorno = e.args[0]
+        else:
+            self.errorno = None
+        self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)
+
+
+class FSEventTestBasic(FileTestCase):
+
+    def tearDown(self):
         try:
             os.remove(TEST_FILE2)
         except OSError:
             pass
+        super(FSEventTestBasic, self).tearDown()
 
     def on_fsevent_cb(self, handle, filename, events, errorno):
+        handle.stop()
         handle.close()
         self.filename = filename
         self.events = events
@@ -899,37 +875,34 @@ class FSEventTestBasic(unittest2.TestCase):
         self.errorno = None
         self.events = None
         self.filename = None
-        fs_event = pyuv.fs.FSEvent(self.loop, TEST_FILE, self.on_fsevent_cb, 0)
+        fs_event = pyuv.fs.FSEvent(self.loop)
+        fs_event.start(TEST_FILE, 0, self.on_fsevent_cb)
         timer = pyuv.Timer(self.loop)
         timer.start(self.timer_cb, 1, 0)
         self.loop.run()
         self.assertEqual(self.errorno, None)
-        self.assertTrue(self.filename == None or self.filename == TEST_FILE)
+        self.assertTrue(self.filename in (None, TEST_FILE, TEST_FILE2))
         self.assertTrue(self.events in (pyuv.fs.UV_CHANGE, pyuv.fs.UV_RENAME))
 
 
-class FSEventTest(unittest2.TestCase):
+class FSEventTest(FileTestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(FSEventTest, self).setUp()
         os.mkdir(TEST_DIR, 0o755)
         with open(os.path.join(TEST_DIR, TEST_FILE), 'w') as f:
             f.write("test")
-        with open(TEST_FILE, 'w') as f:
-            f.write("test")
 
     def tearDown(self):
         shutil.rmtree(TEST_DIR)
-        try:
-            os.remove(TEST_FILE)
-        except OSError:
-            pass
         try:
             os.remove(TEST_FILE2)
         except OSError:
             pass
+        super(FSEventTest, self).tearDown()
 
     def on_fsevent_cb(self, handle, filename, events, errorno):
+        handle.stop()
         handle.close()
         self.filename = filename
         self.events = events
@@ -943,7 +916,8 @@ class FSEventTest(unittest2.TestCase):
         self.errorno = None
         self.events = None
         self.filename = None
-        fs_event = pyuv.fs.FSEvent(self.loop, TEST_DIR, self.on_fsevent_cb, 0)
+        fs_event = pyuv.fs.FSEvent(self.loop)
+        fs_event.start(TEST_DIR, 0, self.on_fsevent_cb)
         timer = pyuv.Timer(self.loop)
         timer.start(self.timer_cb2, 1, 0)
         self.loop.run()
@@ -959,7 +933,8 @@ class FSEventTest(unittest2.TestCase):
         self.errorno = None
         self.events = None
         self.filename = None
-        fs_event = pyuv.fs.FSEvent(self.loop, os.path.join(TEST_DIR, TEST_FILE), self.on_fsevent_cb, 0)
+        fs_event = pyuv.fs.FSEvent(self.loop)
+        fs_event.start(os.path.join(TEST_DIR, TEST_FILE), 0, self.on_fsevent_cb)
         timer = pyuv.Timer(self.loop)
         timer.start(self.timer_cb3, 1, 0)
         self.loop.run()
@@ -968,16 +943,14 @@ class FSEventTest(unittest2.TestCase):
         self.assertTrue(self.events & pyuv.fs.UV_CHANGE)
 
 
-class FSPollTest(unittest2.TestCase):
-
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+class FSPollTest(TestCase):
 
     def tearDown(self):
         try:
             os.remove(TEST_FILE)
         except OSError:
             pass
+        super(FSPollTest, self).tearDown()
 
     def _touch_file(self):
         with open(TEST_FILE, 'w+') as f:
@@ -1011,11 +984,10 @@ class FSPollTest(unittest2.TestCase):
         self.poll_cb_called = 0
         self.timer = pyuv.Timer(self.loop)
         self.fs_poll = pyuv.fs.FSPoll(self.loop)
-        self.fs_poll.start(TEST_FILE, self.on_fspoll, 0.1)
+        self.fs_poll.start(TEST_FILE, 0.1, self.on_fspoll)
         self.loop.run()
         self.assertEqual(self.poll_cb_called, 5)
 
 
 if __name__ == '__main__':
-    unittest2.main(verbosity=2)
-
+    unittest.main(verbosity=2)
