@@ -5,47 +5,99 @@ try:
 except ImportError:
     pwd = None
 import sys
+import unittest
 
-from common import platform_skip, unittest2
-import common
+from common import linesep, platform_skip, TestCase
 import pyuv
 
 
-class ProcessTest(unittest2.TestCase):
+class ProcessTest(TestCase):
 
     def test_process_basic(self):
         self.exit_cb_called = 0
         self.close_cb_called = 0
         def proc_close_cb(proc):
-            self.close_cb_called +=1
+            self.close_cb_called += 1
         def proc_exit_cb(proc, exit_status, term_signal):
             self.assertEqual(exit_status, 0)
             self.exit_cb_called += 1
             proc.close(proc_close_cb)
-        loop = pyuv.Loop.default_loop()
-        proc = pyuv.Process(loop)
-        proc.spawn(file=sys.executable, args=["proc_basic.py"],
-                   exit_callback=proc_exit_cb)
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=[sys.executable, "proc_basic.py"],
+                                  exit_callback=proc_exit_cb)
         pid = proc.pid
-        loop.run()
+        self.loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 1)
         self.assertNotEqual(pid, None)
 
+    def test_process_close(self):
+        proc = pyuv.Process(self.loop)
+        self.assertRaises(RuntimeError, proc.close)
+
+    def test_process_noargs(self):
+        self.exit_cb_called = 0
+        self.close_cb_called = 0
+        def handle_close_cb(proc):
+            self.close_cb_called += 1
+        def proc_exit_cb(proc, exit_status, term_signal):
+            self.exit_cb_called += 1
+            proc.close(handle_close_cb)
+        exe = 'ls' if not os.name == 'nt' else 'dir'
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=[exe],
+                                  exit_callback=proc_exit_cb)
+        self.loop.run()
+        self.assertEqual(self.exit_cb_called, 1)
+        self.assertEqual(self.close_cb_called, 1)
+
+    def test_process_bytesargs(self):
+        self.exit_cb_called = 0
+        self.close_cb_called = 0
+        def handle_close_cb(proc):
+            self.close_cb_called += 1
+        def proc_exit_cb(proc, exit_status, term_signal):
+            self.exit_cb_called += 1
+            proc.close(handle_close_cb)
+        exe = b'ls' if not os.name == 'nt' else b'dir'
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=exe,
+                                  exit_callback=proc_exit_cb)
+        self.loop.run()
+        self.assertEqual(self.exit_cb_called, 1)
+        self.assertEqual(self.close_cb_called, 1)
+
+    def test_process_executable(self):
+        self.exit_cb_called = 0
+        self.close_cb_called = 0
+        def handle_close_cb(proc):
+            self.close_cb_called += 1
+        def proc_exit_cb(proc, exit_status, term_signal):
+            self.exit_cb_called += 1
+            proc.close(handle_close_cb)
+        exe = 'ls' if not os.name == 'nt' else 'dir'
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=[exe],
+                                  executable=exe,
+                                  exit_callback=proc_exit_cb)
+        self.loop.run()
+        self.assertEqual(self.exit_cb_called, 1)
+        self.assertEqual(self.close_cb_called, 1)
+
     def test_process_cwd(self):
         self.exit_cb_called = 0
         self.close_cb_called = 0
         def proc_close_cb(proc):
-            self.close_cb_called +=1
+            self.close_cb_called += 1
         def proc_exit_cb(proc, exit_status, term_signal):
             self.assertEqual(exit_status, 0)
             self.exit_cb_called += 1
             proc.close(proc_close_cb)
-        loop = pyuv.Loop.default_loop()
-        proc = pyuv.Process(loop)
-        proc.spawn(file=sys.executable, args=["proc_basic.py"],
-                   exit_callback=proc_exit_cb, cwd=".")
-        loop.run()
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=[sys.executable, "proc_basic.py"],
+                                  exit_callback=proc_exit_cb,
+                                  cwd=".")
+        self.loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 1)
 
@@ -54,7 +106,7 @@ class ProcessTest(unittest2.TestCase):
         self.close_cb_called = 0
         self.received_output = None
         def handle_close_cb(handle):
-            self.close_cb_called +=1
+            self.close_cb_called += 1
         def proc_exit_cb(proc, exit_status, term_signal):
             self.assertEqual(exit_status, 0)
             self.exit_cb_called += 1
@@ -63,16 +115,16 @@ class ProcessTest(unittest2.TestCase):
             if data is not None:
                 self.received_output = data.strip()
             handle.close(handle_close_cb)
-        loop = pyuv.Loop.default_loop()
-        stdout_pipe = pyuv.Pipe(loop)
+        stdout_pipe = pyuv.Pipe(self.loop)
         stdio = []
         stdio.append(pyuv.StdIO(flags=pyuv.UV_IGNORE))
         stdio.append(pyuv.StdIO(stream=stdout_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_WRITABLE_PIPE))
-        proc = pyuv.Process(loop)
-        proc.spawn(file=sys.executable, args=["proc_stdout.py"],
-                   exit_callback=proc_exit_cb, stdio=stdio)
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=["python", "proc_stdout.py"],
+                                  exit_callback=proc_exit_cb,
+                                  stdio=stdio)
         stdout_pipe.start_read(stdout_read_cb)
-        loop.run()
+        self.loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 2)
         self.assertEqual(self.received_output, b"TEST")
@@ -82,7 +134,7 @@ class ProcessTest(unittest2.TestCase):
         self.close_cb_called = 0
         self.received_output = None
         def handle_close_cb(handle):
-            self.close_cb_called +=1
+            self.close_cb_called += 1
         def proc_exit_cb(proc, exit_status, term_signal):
             self.assertEqual(exit_status, 0)
             self.exit_cb_called += 1
@@ -90,16 +142,16 @@ class ProcessTest(unittest2.TestCase):
         def stdout_read_cb(handle, data, error):
             self.received_output = data.strip()
             handle.close(handle_close_cb)
-        loop = pyuv.Loop.default_loop()
-        stdout_pipe = pyuv.Pipe(loop)
+        stdout_pipe = pyuv.Pipe(self.loop)
         stdio = []
         stdio.append(pyuv.StdIO(flags=pyuv.UV_IGNORE))
         stdio.append(pyuv.StdIO(stream=stdout_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_WRITABLE_PIPE))
-        proc = pyuv.Process(loop)
-        proc.spawn(file=sys.executable, args=["proc_args_stdout.py", "TEST"],
-                   exit_callback=proc_exit_cb, stdio=stdio)
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=[sys.executable, "proc_args_stdout.py", "TEST"],
+                                  exit_callback=proc_exit_cb,
+                                  stdio=stdio)
         stdout_pipe.start_read(stdout_read_cb)
-        loop.run()
+        self.loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 2)
         self.assertEqual(self.received_output, b"TEST")
@@ -109,7 +161,7 @@ class ProcessTest(unittest2.TestCase):
         self.close_cb_called = 0
         self.received_output = None
         def handle_close_cb(handle):
-            self.close_cb_called +=1
+            self.close_cb_called += 1
         def proc_exit_cb(proc, exit_status, term_signal):
             self.assertEqual(exit_status, 0)
             self.exit_cb_called += 1
@@ -117,17 +169,17 @@ class ProcessTest(unittest2.TestCase):
         def stdout_read_cb(handle, data, error):
             self.received_output = data.strip()
             handle.close(handle_close_cb)
-        loop = pyuv.Loop.default_loop()
-        stdout_pipe = pyuv.Pipe(loop)
+        stdout_pipe = pyuv.Pipe(self.loop)
         stdio = []
         stdio.append(pyuv.StdIO(flags=pyuv.UV_IGNORE))
         stdio.append(pyuv.StdIO(stream=stdout_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_WRITABLE_PIPE))
-        proc = pyuv.Process(loop)
-        proc.spawn(file=sys.executable, args=["proc_env_stdout.py"],
-                   env={"TEST": "TEST"}, exit_callback=proc_exit_cb,
-                   stdio=stdio)
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=[sys.executable, "proc_env_stdout.py"],
+                                  env={"TEST": "TEST"},
+                                  exit_callback=proc_exit_cb,
+                                  stdio=stdio)
         stdout_pipe.start_read(stdout_read_cb)
-        loop.run()
+        self.loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 2)
         self.assertEqual(self.received_output, b"TEST")
@@ -139,7 +191,7 @@ class ProcessTest(unittest2.TestCase):
         self.exit_status = -1
         self.term_signal = 0
         def handle_close_cb(handle):
-            self.close_cb_called +=1
+            self.close_cb_called += 1
         def proc_exit_cb(proc, exit_status, term_signal):
             self.exit_cb_called += 1
             self.exit_status = exit_status
@@ -151,18 +203,18 @@ class ProcessTest(unittest2.TestCase):
             handle.close(handle_close_cb)
         def stdin_write_cb(handle, error):
             handle.close(handle_close_cb)
-        loop = pyuv.Loop.default_loop()
-        stdin_pipe = pyuv.Pipe(loop)
-        stdout_pipe = pyuv.Pipe(loop)
+        stdin_pipe = pyuv.Pipe(self.loop)
+        stdout_pipe = pyuv.Pipe(self.loop)
         stdio = []
         stdio.append(pyuv.StdIO(stream=stdin_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_READABLE_PIPE))
         stdio.append(pyuv.StdIO(stream=stdout_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_WRITABLE_PIPE))
-        proc = pyuv.Process(loop)
-        proc.spawn(file=sys.executable, args=["proc_stdin_stdout.py"],
-                   exit_callback=proc_exit_cb, stdio=stdio)
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=[sys.executable, "proc_stdin_stdout.py"],
+                                  exit_callback=proc_exit_cb,
+                                  stdio=stdio)
         stdout_pipe.start_read(stdout_read_cb)
-        stdin_pipe.write(b"TEST"+common.linesep, stdin_write_cb)
-        loop.run()
+        stdin_pipe.write(b"TEST"+linesep, stdin_write_cb)
+        self.loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 3)
         self.assertEqual(self.received_output, b"TEST")
@@ -173,7 +225,7 @@ class ProcessTest(unittest2.TestCase):
         self.exit_status = -1
         self.term_signal = 0
         def handle_close_cb(proc):
-            self.close_cb_called +=1
+            self.close_cb_called += 1
         def proc_exit_cb(proc, exit_status, term_signal):
             self.exit_cb_called += 1
             self.exit_status = exit_status
@@ -182,13 +234,12 @@ class ProcessTest(unittest2.TestCase):
         def timer_cb(timer):
             timer.close(handle_close_cb)
             proc.kill(15)
-        loop = pyuv.Loop.default_loop()
-        timer = pyuv.Timer(loop)
+        timer = pyuv.Timer(self.loop)
         timer.start(timer_cb, 0.1, 0)
-        proc = pyuv.Process(loop)
-        proc.spawn(file=sys.executable, args=["proc_infinite.py"],
-                   exit_callback=proc_exit_cb)
-        loop.run()
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=[sys.executable, "proc_infinite.py"],
+                                  exit_callback=proc_exit_cb)
+        self.loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 2)
         if sys.platform == 'win32':
@@ -202,7 +253,7 @@ class ProcessTest(unittest2.TestCase):
         self.exit_cb_called = 0
         self.close_cb_called = 0
         def proc_close_cb(proc):
-            self.close_cb_called +=1
+            self.close_cb_called += 1
         def proc_exit_cb(proc, exit_status, term_signal):
             self.assertEqual(exit_status, 0)
             self.exit_cb_called += 1
@@ -211,11 +262,14 @@ class ProcessTest(unittest2.TestCase):
             self.skipTest("test disabled if running as non-root")
             return
         p_info = pwd.getpwnam("nobody")
-        loop = pyuv.Loop.default_loop()
-        proc = pyuv.Process(loop)
-        proc.spawn(file="./proc_basic.py", exit_callback=proc_exit_cb, uid=p_info.pw_uid, gid=p_info.pw_gid, flags=pyuv.UV_PROCESS_SETUID|pyuv.UV_PROCESS_SETGID)
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=[sys.executable, "proc_basic.py"],
+                                  exit_callback=proc_exit_cb,
+                                  uid=p_info.pw_uid,
+                                  gid=p_info.pw_gid,
+                                  flags=pyuv.UV_PROCESS_SETUID|pyuv.UV_PROCESS_SETGID)
         pid = proc.pid
-        loop.run()
+        self.loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 1)
         self.assertNotEqual(pid, None)
@@ -225,7 +279,7 @@ class ProcessTest(unittest2.TestCase):
         self.exit_cb_called = 0
         self.close_cb_called = 0
         def proc_close_cb(proc):
-            self.close_cb_called +=1
+            self.close_cb_called += 1
         def proc_exit_cb(proc, exit_status, term_signal):
             self.assertNotEqual(exit_status, 0)
             self.exit_cb_called += 1
@@ -233,10 +287,12 @@ class ProcessTest(unittest2.TestCase):
         if os.getuid() != 0:
             self.skipTest("test disabled if running as non-root")
             return
-        loop = pyuv.Loop.default_loop()
-        proc = pyuv.Process(loop)
-        proc.spawn(file="./proc_basic.py", exit_callback=proc_exit_cb, uid=-42424242, flags=pyuv.UV_PROCESS_SETUID)
-        loop.run()
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=[sys.executable, "proc_basic.py"],
+                                  exit_callback=proc_exit_cb,
+                                  uid=-42424242,
+                                  flags=pyuv.UV_PROCESS_SETUID)
+        self.loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 1)
 
@@ -244,13 +300,13 @@ class ProcessTest(unittest2.TestCase):
         self.exit_cb_called = 0
         def proc_exit_cb(proc, exit_status, term_signal):
             self.exit_cb_called += 1
-        loop = pyuv.Loop.default_loop()
-        proc = pyuv.Process(loop)
-        proc.spawn(file=sys.executable, args=["proc_basic.py"],
-                   exit_callback=proc_exit_cb, flags=pyuv.UV_PROCESS_DETACHED)
-        proc.unref()
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=[sys.executable, "proc_basic.py"],
+                                  exit_callback=proc_exit_cb,
+                                  flags=pyuv.UV_PROCESS_DETACHED)
+        proc.ref = False
         pid = proc.pid
-        loop.run()
+        self.loop.run()
         self.assertEqual(self.exit_cb_called, 0)
         proc.kill(0)
         proc.kill(15)
@@ -258,4 +314,4 @@ class ProcessTest(unittest2.TestCase):
 
 
 if __name__ == '__main__':
-    unittest2.main(verbosity=2)
+    unittest.main(verbosity=2)
