@@ -1,28 +1,27 @@
 
+import unittest
 import weakref
 
-from common import unittest2
+from common import TestCase
 import pyuv
 
 
-class WeakrefTest(unittest2.TestCase):
+class WeakrefTest(TestCase):
     handle_types = ('Check', 'Idle', 'Pipe', 'Prepare', 'TCP', 'Timer', 'UDP')
 
     def test_weakref(self):
-        loop = pyuv.Loop()
         refs = []
         for type in self.handle_types:
             klass = getattr(pyuv, type)
-            obj = klass(loop)
+            obj = klass(self.loop)
             refs.append(weakref.ref(obj))
             del obj
         for ref in refs:
             self.assertNotEqual(ref(), None)
-        loop.run()
+        self.loop.run()
         for ref in refs:
             self.assertEqual(ref(), None)
 
 
 if __name__ == '__main__':
-    unittest2.main(verbosity=2)
-
+    unittest.main(verbosity=2)
